/*
  ------------------------------------------------------------------------------

    Copyright © 2005-2013, Tommi T. Järvi
                           firstname . jarvi @ iki . fi (remove spaces)


    This file is part of one of my programs.

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2,
    as published by the Free Software Foundation.

    This is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  ------------------------------------------------------------------------------
*/


#include "hydrogenbond.h"


void H_Bond::analyze(Atom_System &s, double anglemin, double anglemax, double cut13, double cut12_23, int t1, int t2, int t3, double time) {

  /*


    anglemin - angle cutoff, minimum (radians)
    anglemin - angle cutoff, maximum (radians)
    cut13    - cutoff for bond 1-3
    cut12_23 - cutoff for bonds 1-2 and 2-3
    t1,t2,t3 - types required for angle t1-t2-t3
    time     - time for output
  */


  // --- variables

  int i, j, k, m, n, l, o;  // loops, temps
  int maxpairs = 100;  // max number of neareast neighbours
  int maxcommnei = 10;  // max number of common neighbours
  int ncommnei;  // number of common neighbours of current atom
  bool take;  // this neighbour already there?
  valarray<int> commnei;  // common neighbours
  int natoms;
  int nhbond;  // number of matching bonds found per atom pair
  double angle;  // current angle
  int totalbonds;  // total number of bonds found


  // --- check
  if(anglemin < 0 || anglemin > M_PI || anglemax < 0 || anglemax > M_PI) {
    cout << "H_Bond::analyze(): Error: Invalied angle range " << anglemin << " - " << anglemax << endl;
    exit(-1);
  }


  // --- report
  cout << "H_Bond::analyze(): Analyzing (hydrogen) bonds..." << endl;
  cout << "H_Bond::analyze(): Angle range: " << anglemin << " - " << anglemax << " (radians)" << endl;
  cout << "H_Bond::analyze(): Atom types: " << t1 << "-" << t2 << "-" << t3 << " = " << s.atypes.name(t1) << "-" << s.atypes.name(t2) << "-" << s.atypes.name(t3) << endl;
  cout << "H_Bond::analyze(): Outputting: hbond time at1 at2 at3 r(13) r(12) r(23) angle(123) (angle in degrees [0,180])" << endl;


  // --- init arrays
  natoms = s.atoms();
  commnei.resize(maxcommnei);


  // --- calculate
  totalbonds = 0;

  // loop over atoms
  for(i=0;i<natoms;i++) {

    // loop over other atoms
    for(o=1;o<=s.symmnlist[i][0];o++) {
      j = s.symmnlist[i][o];

      // apply cutoff
      if(s.dr(i,j) > cut13) continue;

      // take only the right angles
      if( (s.atype[i] == t1 && s.atype[j] == t3) || (s.atype[i] == t3 && s.atype[j] == t1) ) {
	// ok
      }
      else {
	continue;
      }

      // take each angle only once (t1-t2-t3 = t3-t2-t1)
      if(j < i) continue;

      // don't take atoms with themselves
      if(i==j) {
	cerr << "H_Bond::analyze(): Error! Bug, report it." << endl;
	exit(-1);
      }

      // --- find common neighbours

      // resize common neighbour array if needed
      if(s.symmnlist[i][0] < s.symmnlist[j][0]) k = s.symmnlist[j][0];
      else k = s.symmnlist[i][0];
      if(maxcommnei < k) {
	maxcommnei = k;
	commnei.resize(maxcommnei);
      }

      // loop over neighbours of atom i
      ncommnei = 0;
      for(k=1;k<=s.symmnlist[i][0];k++) {

	// loop over neighbours of atom j
	for(m=1;m<=s.symmnlist[j][0];m++) {

	  // take neighbour
	  if(s.symmnlist[i][k]==s.symmnlist[j][m]) {
	    take = true;

            // don't take i or j as a common neighbour of i and j
            if(s.symmnlist[i][k]==i || s.symmnlist[i][k]==j) {
              cerr << "H_Bond::analyze(): Error / Bug: Atom is its own neighbour." << endl;
              exit(-1);
            }

            // check if this neighbour is listed already
	    for(n=0;n<ncommnei;n++) {
	      if(commnei[n]==s.symmnlist[i][k]) take = false;
	    }

            // take
	    if(take) {
	      commnei[ncommnei] = s.symmnlist[i][k];
	      ncommnei++;
	    }
	  }  // neighbour taken

	}  // end of loop over neighbours of atom j
      }  // end of loop over neighbours of atom i


      // --- skip for atoms with no common neighbours
      if(ncommnei==0) continue;


      // --- common neighbours found, check for (hydrogen) bond

      nhbond = 0;
      for(k=0;k<ncommnei;k++) {
	int m = commnei[k];  // middle atom: angle i-m-j

	// check
	if(i==m || j==m) {
	  cerr << "H_Bond::analyze(): Error / Bug: An atom is its own common neighbour." << endl;
	  exit(-1);
	}

	// check for type
	if(s.atype[m]!=t2) continue;

	// possible (hydrogen) bond, check angle and bonds
	angle = s.angle(i, m, j);
	if(angle > anglemin && angle < anglemax && s.dr(i,m) < cut12_23 && s.dr(m,j) < cut12_23) {
	  // bond found
	  nhbond++;
	  totalbonds++;
	  cout << "hbond " << time << " " << i << " " << m << " " << j << " " <<  s.dr(i, j) << " " << s.dr(i,m) << " " << s.dr(j,m) << " " << angle*180.0/M_PI << endl;

	  // check once more
	  if(s.atype[i] != t1 && s.atype[i] != t3) {
	    cerr << "H_Bond::analyze(): Error / Bug: Please report." << endl;
	    exit(-1);
	  }
	  if(s.atype[j] != t1 && s.atype[j] != t3) {
	    cerr << "H_Bond::analyze(): Error / Bug: Please report." << endl;
	    exit(-1);
	  }
	}
      }

      // check for multiple bonds between same atom
      if(nhbond >= 2) {
	cerr << "H_Bond::analyze(): Warning: Multiple (hydrogen) bonds between atoms " << i << " and " << j << endl;
      }

    }  // end of loop over other atoms
  }  // end of loop over atoms


  // --- report
  cout << "H_Bond::analyze(): Done. Total number of (hydrogen) bonds found: " << totalbonds << endl;
}
