/*
  ------------------------------------------------------------------------------

    Copyright © 2005-2013, Tommi T. Järvi
                           firstname . jarvi @ iki . fi (remove spaces)


    This file is part of one of my programs.

    This is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2,
    as published by the Free Software Foundation.

    This is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this file; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

  ------------------------------------------------------------------------------
*/


#include <iostream>
#include <fstream>
#include <cassert>
#include "atomsystem.h"
#include "cnn.h"
#include "hydrogenbond.h"
#include "mathsubs.h"
#include "licence.h"
#include <gsl/gsl_matrix.h>  // used by qsolid
#include <gsl/gsl_complex.h>  // used by qsolid
#include <gsl/gsl_complex_math.h>  // used by qsolid


using namespace std;


int main(int argc, char *argv[]) {


  // --- variables

  int i;  // loop over command line arguments
  int frame;  // current frame
  int frames;  // number of frames
  Atom_System *system;  // system
  string infile;  // input file name
  string arg;
  int xyzin = 0;  // xyzmodes
  valarray<int> pbc(3);  // pbc's
  valarray<double> box(3);  // box, needed by pbc
  ifstream afile;  // input file, used to count frames
  double time;  // time of frame
  int i1;  // temp
  double t1;  // temp
  string mode;  // input mode (read file / create etc.)
  int istart;  // first command argument on command line
  int istartbeg;  // reset to this between frames
  valarray<int> cells;  // unit cells to create


  // --- functions
  void commandloop(Atom_System *system, int frame, int frames, int xyzin, double time, int istart, int argc, char *argv[], string mode, bool dummy_check = false);
  void argcheck(int argc, int i, int req);  // check that there have been enough arguments


  // --- copyright
  cout << endl;
  cout << " Euler" << endl;
  cout << "   svn r276 (2013-04-19)" << endl;
  cout << "   (C) 2005-2013 Tommi T. Järvi" << endl;
  cout << "       (mainly, see option 'gpl')" << endl;
  cout << endl;
  cout << "Euler has been made mainly for my own use, so it comes with no" << endl;
  cout << "warranty and you should always be suspicious of the output." << endl;
  cout << "This is free software, and you are welcome to redistribute it" << endl;
  cout << "under certain conditions. Run with argument 'gpl' for details." << endl;
  cout << endl;


  // --- licence
  if(argc==2) {
    arg = argv[1];
    if(arg=="gpl") {
      cout << "************************" << endl;
      cout << " Euler is licenced under the GNU General Public Licence, version 2." << endl;
      cout << "The complete licence is shown below. All files include a copyright" << endl;
      cout << "notice and all but the files associated with the Mersenne Twister RNG" << endl;
      cout << "are (C) Tommi Järvi except if stated otherwise. The licence for" << endl;
      cout << "Mersenne Twister is shown in the respective files (mt19937ar.*) and" << endl;
      cout << "below after the GNU GPL." << endl;
      cout << "************************" << endl;
      Licence::gpl();
      cout << "************************" << endl;
      Licence::mt();
      cout << "************************" << endl;
    }
    exit(0);
  }


  // --- command line arguments, initialisation, and calculation

  if(argc == 1) {
    cout << "" << endl;
    cout << "Usage: " << argv[0] << " <input system> [options] [commands]" << endl;
    cout << "" << endl;
    cout << "  The commands are done on the input system in the order they are given." << endl;
    cout << "  The input system can be specified by the following methods:" << endl;
    cout << "    xyz <file> <mode>                      Read in an xyz-file" << endl;
    cout << "    lmpcdump <file>                        Read in LAMMPS \"custom dump\" file" << endl;
    cout << "    lmpdata <file> <mode>                  Rean in LAMMPS \"data\" file" << endl;
    cout << "    basis[v] <file> <mode> <f> <nx ny nz>  Read basis file and create system" << endl;
    cout << "                                           (for basisv with velocities)" << endl;
    cout << "" << endl;
    cout << "  See man-page for details." << endl;
    cout << "" << endl;
    cout << "  commands & options            explanation" << endl;
    cout << "  ----------------------------------------------" << endl;
    cout << "" << endl;
    cout << "  --- options ---" << endl;
    cout << "  pbc <x y z>                 pbc's, 0 for no, 1 for yes" << endl;
    cout << "  box <x y z>                 box size (cannot be specified for system created from basis)" << endl;
    cout << "" << endl;
    cout << "  --- commands - basic information ---" << endl;
    cout << "  temperature                   Get temperature and other stuff" << endl;
    cout << "  spheret <cut>                 Replace kinetic energy of each atom with average within cut." << endl;
    cout << "  limits                        Get extremal coordinates" << endl;
    cout << "  cmekin                        Output center of mass velocity, kinetic energy etc." << endl;
    cout << "  masses                        Total mass etc." << endl;
    cout << "" << endl;
    cout << "  --- commands - units etc. ---" << endl;
    cout << "  unit_l <f>                    Multiply units of length by f." << endl;
    cout << "  unit_t <f>                    Multiply units of time by f." << endl;
    cout << "  rename_type <i name>          Rename atom type i (integer) to name (string)" << endl;
    cout << "" << endl;
    cout << "  --- commands - control ---" << endl;
    cout << "  df <i>                        Only take every i'th frame (but always first and last)." << endl;
    cout << "  begint <t>                    Discard frames with time smaller than t." << endl;
    cout << "  endt <t>                      Discard frames with time larger than t." << endl;
    cout << "  dt <t>                        Only take frames at intervals t." << endl;
    cout << "" << endl;
    cout << "  --- commands - basic transformations ---" << endl;
    cout << "  center                        Center system" << endl;
    cout << "  centerv                       Center cm-velocity" << endl;
    cout << "  rotate <phi theta psi>        Euler rotation" << endl;
    cout << "  rotateaxis <phi theta delta>  Rotate around axis phi, theta by delta" << endl;
    cout << "  rotatevec <x y z delta>       Rotate around axis x, y, z by delta" << endl;
    cout << "  translate <x y z>             Translate (fixed atoms too)" << endl;
    cout << "  movepos                       Move system so that all atoms have non-negative coordinates" << endl;
    cout << "  scale <f>                     Scale coordinates by f. Don't touch box size etc." << endl;
    cout << "  addek <E>                     Add E eV/at. of kinetic energy to the system by scaling velocities." << endl;
    cout << "  addveldz <vel z>              Add velocity (vel in Å/fs) to atoms above z in the z-direction. (negative = downwards)" << endl;
    cout << "  moveatom <n x y z>            Move atom n" << endl;
    cout << "  removeat <n mode>             Remove atom n [0,#atoms-1]" << endl;
    cout << "  removetype <n>                Remove atoms of type n (by integer)" << endl;
    cout << "  removename <Xx>               Remove atoms of type Xx (by element name)" << endl;
    cout << "  removerand <n>                Remove n atoms randomly" << endl;
    cout << "  removeV <a b>                 Remove atoms with V in ]a,b[" << endl;
    cout << "  moveV <a b x y z>             Move atoms with V in ]a,b[ by [x,y,z]" << endl;
    cout << "  removeK <a b>                 Remove atoms with K in ]a,b[" << endl;
    cout << "  remove <dir 1/0 cut>          Remove atoms above (1) or below (0) cut in dir {0,1,2}." << endl;
    cout << "  removehs <bx by bz dx dy dz>  Remove atoms with (r-b) \\cdot d > 0" << endl;
    cout << "  removedist <cut x y z>        Remove atoms farther away than cut from (x,y,z)" << endl;
    cout << "  removevor <x1 y1 z1 x2 y2 z2> Remove atoms closer to v2 than v1" << endl;
    cout << "  removecoord <cut min max>     Remove atoms with coordination number between min and max" << endl;
    cout << "  removedat <cut n min>         Remove atoms farther away than cut from atom n (don't go below min atoms)" << endl;
    cout << "  removetc <cut>                Remove atoms so that no atoms are closer to each other than cut" << endl;
    cout << "  removecurve <...>             Remove atoms above or below a curve" << endl;
    cout << "  join <file>                   Join a system from file to this system" << endl;
    cout << "  ontop <dist file>             Join a system from file on top of this system" << endl;
    cout << "  ensuredist <cut>              Ensure no atoms are closer than cut to each other" << endl;
    cout << "" << endl;
    cout << "  --- commands - clustering ---" << endl;
    cout << "  cluster <cut>                 Cluster atoms with cutoff" << endl;
    cout << "  leavelargest                  Remove atoms not in the largest block." << endl;
    cout << "  removelargest                 Remove atoms in the largest block." << endl;
    cout << "  removeblocksizes <n1 n2>      Remove blocks with n1-n2 atoms." << endl;
    cout << "  cldist                        Output cluster size distributions and statistics." << endl;
    cout << "" << endl;
    cout << "  --- commands - analysis ---" << endl;
    cout << "  getmoved <rcut file>          Compares to another xyz-file and only outputs moved atoms" << endl;
    cout << "  getcenter                     Try to figure out geometric center of atoms within a periodic system" << endl;
    cout << "  sro <a at nei zmin zmax>      SRO-parameter" << endl;
    cout << "  ssro <a at nei>               Smart SRO-parameter" << endl;
    cout << "  lsro <a at nei>               Local smart SRO-parameter" << endl;
    cout << "  sfactor <a nx ny nz>          Stucture factor with Miller indices n" << endl;
    cout << "  sfactorz <a zlim nx ny nz>    Stucture factor of atoms above zlim normalised to those below" << endl;
    cout << "  pair a b dr                   Get pair correlation function with r=[a,b]" << endl;
    cout << "  art                           Output amplitude for R(t), where R is the distance of an atom from the system cm" << endl;
    cout << "  disp                          Get displacements relative to first frame" << endl;
    cout << "  cna <cut mode>                Common neighbour analysis" << endl;
    cout << "  hbond <cut13 cut12_23 a b t1 t2 t3> Hydrogen bond analysis" << endl;
    cout << "  qsolid <cut thres frac l>     Get solid/liquid atoms based on q-correlation" << endl;
    cout << "  qcorr <cut l>                 Get q-correlation number for all pairs" << endl;
    cout << "  coordination <cut>            Get number of neighbours (inside cut) for each atom + statistics" << endl;
    cout << "  bdef <...>                    Bond analysis, see man-page for description" << endl;
    cout << "  mdef <...>                    Molecule analysis, see man-page for description" << endl;
    cout << "  findvac <r dr>                Find vacancies by inserting spheres" << endl;
    cout << "  afm_tip <file dx cut nlf>     Simulated AFM image convoluted by tip shape" << endl;
    cout << "" << endl;
    cout << " --- commands - miscellaneous ---" << endl;
    cout << "  distz <x y>                   Get distance of closest atom from line (x,y) || z" << endl;
    cout << "  distfrom <x y z>              Get distance of each atom from point (x,y,z)" << endl;
    cout << "" << endl;
    cout << "  runaveV <dir a b dx delta>    Get running average of the V-array with bins of width delta." << endl;
    cout << endl;
    cout << "  --- output ---" << endl;
    cout << "" << endl;
    cout << "  pad <n x y z>                 Insert atoms of type \"Pp\" at <x,y,z> so that total number of atoms is n." << endl;
    cout << "  savexyz <file mode>           Save xyz-file" << endl;
    cout << "  savexyzfr <file mode>         Save xyz-file of each frame with name fileN.xyz, N in [1,N]" << endl;
    cout << "  savexyzfrc <mode>             Same, except files named as [system comment line].xyz" << endl;
    cout << "  savepov <file m min max>      Save pov-file" << endl;
    cout << "  savepovfr <file m min max>    Save pov-file frames with names fileN.pov" << endl;
    cout << "  saveposcar <file a order>     Save poscar-file (for VASP)" << endl;
    cout << "  saveposcars <file a order>    Save poscar-file with \"Selective dynamics\"(for VASP)" << endl;
    cout << "  savebgf <file mode>           Save bgf-file compatible with ReaxFF." << endl;
    cout << "  savebgfd <file mode desc>     Save bgf-file compatible with ReaxFF." << endl;
    cout << "  savemdcore <file mode>        Save mdcore-compatible file." << endl;
    cout << "  savesiesta <file>             Save Siesta fdf-file" << endl;
    cout << "  savelmp <file> <mode>         Save LAMMPS data file" << endl;
    cout << "  lmp_bonds <id t1 t2 cut>      Save LAMMPS bond information" << endl;
    cout << "  lmp_angles                    -> See lmp_bonds" << endl;
    cout << "  lmp_dihedrals                 -> See lmp_bonds" << endl;
    cout << "" << endl;
    exit(0);
  }


  // --- report command line
  cout << endl;
  cout << "Command line: ";
  for(i=0;i<argc;i++) {
    cout << argv[i] << " ";
  }
  cout << endl;


  // --- set input mode and command line argument counter
  mode = argv[1];
  istart = 1;


  // --- defaults
  time = 0.0;


  // ***************************
  // *** INPUT FROM XYZ-FILE ***
  // ***************************

  if(mode=="xyz") {

    // --- input file and mode

    argcheck(argc, istart, 2);
    infile = argv[istart+1];
    xyzin = atoi(argv[istart+2]);
    cout << "Input file          " << infile << endl;
    cout << "      mode          " << xyzin << endl;
    istart = istart + 3;

    // --- find out number of frames

    afile.open(infile.c_str());
    assert(afile);
    cout << "Determining number of frames in file... (this may take a while)" << endl;

    frames = 0;
    while(!afile.eof()) {
      afile >> i1;
      afile.ignore(999999999, '\n');
      afile.ignore(999999999, '\n');
      for(i=0;i<i1;i++) {
	afile.ignore(999999999, '\n');
      }
      frames++;
    }
    frames--;  // not elegant

    cout << "  frames expected: " << frames << endl;

    afile.close();
    afile.clear();

    // --- loop over frames

    cout << "Opening file " << infile << endl;
    afile.open(infile.c_str());
    assert(afile);

    istartbeg = istart;
    for(frame=1;frame<=frames;frame++) {

      // make sure all parameters are read in
      istart = istartbeg;

      // init system
      system = new Atom_System();

      // reset some parameters (defaults)
      pbc = 0;
      box = 0;
      system->setpbc(pbc);
      system->setbox(box);

      // read file
      system->readxyzframe(afile, xyzin);

      // get time if possible
      {
	bool got;  // got time?
	got = system->comment_time(time);
	if(got) {
	  cout << "Determined frame time: " << time << endl;
	}
	else {
	  time = (double)frame;
	  cout << "Could not determine frame time, using frame number: " << time << endl;
	}
      }

      // get (optional box & pbc)
      if(istart < argc) {
	arg = argv[istart];
	while(arg=="box" || arg=="pbc") {
	  if(arg=="box") {
	    argcheck(argc, istart, 3);
	    box[0] = atof(argv[istart+1]);
	    box[1] = atof(argv[istart+2]);
	    box[2] = atof(argv[istart+3]);
	    istart = istart + 4;
	    cout << "Setting box " << box[0] << " " << box[1] << " " << box[2] << endl;
	    system->setbox(box);
	  }
	  else if(arg=="pbc") {
	    argcheck(argc, istart, 3);
	    pbc[0] = atoi(argv[istart+1]);
	    pbc[1] = atoi(argv[istart+2]);
	    pbc[2] = atoi(argv[istart+3]);
	    istart = istart + 4;
	    cout << "Setting pbc " << pbc[0] << " " << pbc[1] << " " << pbc[2] << endl;
	    system->setpbc(pbc);
	  }
	  arg = argv[istart];
	}
      }

      // process commands
      commandloop(system, frame, frames, xyzin, time, istart, argc, argv, mode, true);
      commandloop(system, frame, frames, xyzin, time, istart, argc, argv, mode);

      // --- done
      delete system;
    }

    afile.close();
  }

  // ******************************************
  // *** INPUT FROM LAMMPS CUSTOM DUMP FILE ***
  // ******************************************

  else if(mode=="lmpcdump") {

    // --- input file and mode

    argcheck(argc, istart, 1);
    infile = argv[istart+1];
    cout << "Input file          " << infile << endl;
    istart = istart + 2;

    // --- find out number of frames

    afile.open(infile.c_str());
    assert(afile);
    cout << "Determining number of frames in file... (this may take a while)" << endl;

    frames = 0;
    while(!afile.eof()) {
      system = new Atom_System();
      system->readlammps_custom_dump_frame(afile, true);
      frames++;
      delete system;
      // check for end of file
      i1 = afile.peek();
      if(i1==EOF) break;
    }

    cout << "  frames expected: " << frames << endl;

    afile.close();
    afile.clear();

    // --- loop over frames

    cout << "Opening file " << infile << endl;
    afile.open(infile.c_str());
    assert(afile);

    istartbeg = istart;
    for(frame=1;frame<=frames;frame++) {

      // make sure all parameters are read in
      istart = istartbeg;

      // init system
      system = new Atom_System();

      // reset some parameters (defaults)
      pbc = 0;
      box = 0;
      system->setpbc(pbc);
      system->setbox(box);

      // read file
      time = system->readlammps_custom_dump_frame(afile);

      // get optional pbc
      if(istart < argc) {
	arg = argv[istart];
	while(arg=="box" || arg=="pbc") {
	  if(arg=="box") {
	    cerr << "Do not give box for a system read from a LAMMPS custom dump file. The size is contained in the file." << endl;
	    exit(-1);
	  }
	  else if(arg=="pbc") {
	    argcheck(argc, istart, 3);
	    pbc[0] = atoi(argv[istart+1]);
	    pbc[1] = atoi(argv[istart+2]);
	    pbc[2] = atoi(argv[istart+3]);
	    istart = istart + 4;
	    cout << "Setting pbc " << pbc[0] << " " << pbc[1] << " " << pbc[2] << endl;
	    system->setpbc(pbc);
	  }
	  if(istart >= argc) break;
	  arg = argv[istart];
	}
      }

      // process commands
      commandloop(system, frame, frames, xyzin, time, istart, argc, argv, mode, true);
      commandloop(system, frame, frames, xyzin, time, istart, argc, argv, mode);

      // --- done
      delete system;
    }

    afile.close();
  }

  // ***********************************
  // *** INPUT FROM LAMMPS DATA FILE ***
  // ***********************************

  else if(mode=="lmpdata") {

    // --- input file and mode

    argcheck(argc, istart, 2);
    infile = argv[istart+1];
    int lmpmode = atoi(argv[istart+2]);
    cout << "Input file          " << infile << endl;
    cout << "      mode          " << lmpmode << endl;
    istart = istart + 3;

    // init system
    system = new Atom_System();

    // reset some parameters (defaults)
    pbc = 0;
    box = 0;
    system->setpbc(pbc);
    system->setbox(box);

    // read file
    system->readlammps_data(infile, lmpmode);

    // get (optional box & pbc)
    if(istart < argc) {
      arg = argv[istart];
      while(arg=="box" || arg=="pbc") {
	if(arg=="box") {
	  cerr << "Do not give box for a system read from a LAMMPS data file. The size is contained in the file." << endl;
	  exit(-1);
	}
	else if(arg=="pbc") {
	  argcheck(argc, istart, 3);
	  pbc[0] = atoi(argv[istart+1]);
	  pbc[1] = atoi(argv[istart+2]);
	  pbc[2] = atoi(argv[istart+3]);
	  istart = istart + 4;
	  cout << "Setting pbc " << pbc[0] << " " << pbc[1] << " " << pbc[2] << endl;
	  system->setpbc(pbc);
	}
	arg = argv[istart];
	}
    }

    // process commands
    frame = 1;
    frames = 1;
    xyzin = 0;
    time = 0.0;
    commandloop(system, frame, frames, xyzin, time, istart, argc, argv, mode, true);
    commandloop(system, frame, frames, xyzin, time, istart, argc, argv, mode);

    // --- done
    delete system;
  }

  // *************************
  // *** CREATE FROM BASIS ***
  // *************************

  else if(mode=="basis" || mode=="basisv") {

    // --- basis file, mode, and size in unit cells
    argcheck(argc, istart, 6);
    infile = argv[istart+1];  // basis file
    i1 = atoi(argv[istart+2]);  // basis file mode
    t1 = atof(argv[istart+3]);  // scaling factor
    cells.resize(3);  // number of unit cells
    cells[0] = atoi(argv[istart+4]);
    cells[1] = atoi(argv[istart+5]);
    cells[2] = atoi(argv[istart+6]);
    istart = istart + 7;

    // --- create system
    system = new Atom_System();
    if(mode=="basis") {
      system->read_unit_cell_and_init(infile, cells, t1, i1, 1);
    }
    else if(mode=="basisv") {
      system->read_unit_cell_and_init(infile, cells, t1, i1, 2);
    }
    else {
      cout << "Encountered stupid bug in reading basis. Fix or report." << endl;
      exit(-1);
    }

    // --- get (optional box & pbc) (no box)
    if(istart < argc) {
      arg = argv[istart];
      while(arg=="box" || arg=="pbc") {
	if(arg=="box") {
	  cerr << "Do not give box for a system that you create." << endl;
	  exit(-1);
	  argcheck(argc, istart, 3);
	  box[0] = atof(argv[istart+1]);
	  box[1] = atof(argv[istart+2]);
	  box[2] = atof(argv[istart+3]);
	  istart = istart + 4;
	  cout << "Setting box " << box[0] << " " << box[1] << " " << box[2] << endl;
	  system->setbox(box);
	}
	else if(arg=="pbc") {
	  argcheck(argc, istart, 3);
	  pbc[0] = atoi(argv[istart+1]);
	  pbc[1] = atoi(argv[istart+2]);
	  pbc[2] = atoi(argv[istart+3]);
	  istart = istart + 4;
	  cout << "Setting pbc " << pbc[0] << " " << pbc[1] << " " << pbc[2] << endl;
	  system->setpbc(pbc);
	}
	arg = argv[istart];
      }
    }

    // --- process commands
    frame = 1;
    frames = 1;
    xyzin = 0;
    time = 0.0;
    commandloop(system, frame, frames, xyzin, time, istart, argc, argv, mode, true);
    commandloop(system, frame, frames, xyzin, time, istart, argc, argv, mode);

    // --- done
    delete system;
  }

  // *************************
  // *** UNKNOWN MODE      ***
  // *************************

  else {
    cerr << "Unknown input mode: " << mode << endl;
    exit(-1);
  }


  return 0;
}


void commandloop(Atom_System *system, int frame, int frames, int xyzin, double time, int istart, int argc, char *argv[], string mode, bool dummy_check) {

  /*
    Process commands given on the command line.

    system                  system to be processes
    frame                   frame should be in [1,nframes]
    frames                  total number of frames
    xyzin                   xyz input mode
    time                    time of frame
    istart                  first command line argument to use
    argc                    number of command line arguments
    argv                    command line
    mode                    read-in mode (xyz, basis, etc.)
    dummy_check             if true, just check number of command line arguments
  */


  // --- variables

  int i;   // loop over command line options

  // --- static variables

  static double lastt = -1;  // for dt to keep up with the last frame processed
  static int lastf = -1;     // for df to keep up with the last frame processed
  static valarray<double> rmin, rmax;  // used for amplitude of R(t)

  static Atom_System prev_frame;  // for storing previous frame for disp

  // --- other command specific variables

  // lammps bonds etc.
  int lmp_bonds = -1;
  int lmp_angles = -1;
  int lmp_dihedrals = -1;
  int lmp_bond_types = 0;
  int lmp_angle_types = 0;
  int lmp_dihedral_types = 0;

  // --- functions
  void argcheck(int argc, int i, int req);  // check that there have been enough arguments


  // --- check
  if(istart >= argc && !dummy_check) {
    cout << "No commands to process. You might want to add at least something like \"savexyz p.xyz 0\"." << endl;
    return;
  }


    // --- loop over command line arguments
    for(i=istart;i<argc;i=i) {
      string arg = argv[i];

      // ******************************
      // error on box / pbc
      // ******************************
      if(arg=="box" || arg=="pbc") {
	cerr << "The periodics (pbc) and box (box) must be specified before any other commands." << endl;
	exit(-1);
      }

      // ******************************
      // df
      // ******************************
      if(arg=="df") {
	int i1;
	bool tb1;

	argcheck(argc, i, 1);
	i1 = atoi(argv[i+1]);  // frame interval
	i = i + 2;
	if(dummy_check) continue;

	// check
	if(i1 <= 0) {
	  cerr << "Error: The frame interval should be positive." << endl;
	  exit(-1);
	}

	// take this frame?
	tb1 = false;
	if(frame==1) tb1 = true;
	if(frame==frames) tb1 = true;
	if(frame - lastf >= i1) tb1 = true;

	// skip or process
	if(tb1) {
	  lastf = frame;
	}
	else {
	  i = argc + 1;
	  break;
	}
      }

      // ******************************
      // dt
      // ******************************
      else if(arg=="dt") {
	double t1;

	argcheck(argc, i, 1);
	t1 = atof(argv[i+1]);  // time interval
	i = i + 2;
	if(dummy_check) continue;

	// check
	if(t1 <= 0) {
	  cerr << "Error: The time interval should be positive." << endl;
	  exit(-1);
	}
	// discard?
	if(frame==1) {
	  // first time in dt
	  lastt = time;
	}
	else {
	  // take
	  if(time - lastt > t1) {
	    lastt = time;
	  }
	  else {
	    i = argc+1;
	    break;
	  }
	}
      }

      // ******************************
      // discard t<
      // ******************************
      else if(arg=="begint") {
	double t1;

	argcheck(argc, i, 1);
	t1 = atof(argv[i+1]);
	i = i + 2;
	if(dummy_check) continue;

	// check
	if(t1 <= 0) {
	  cerr << "Error: The time to begin should be positive." << endl;
	  exit(-1);
	}
	// discard?
	if(time < t1) {
	  i = argc+1;
	  break;
	}
      }

      // ******************************
      // discard t>
      // ******************************
      else if(arg=="endt") {
	double t1;

	argcheck(argc, i, 1);
	t1 = atof(argv[i+1]);
	i = i + 2;
	if(dummy_check) continue;

	// check
	if(t1 <= 0) {
	  cerr << "Error: The time to end should be positive." << endl;
	  exit(-1);
	}
	// discard?
	if(time > t1) {
	  i = argc+1;
	  break;
	}
      }

      // ******************************
      // center
      // ******************************
      else if(arg=="center") {
	argcheck(argc, i, 0);
	i = i + 1;
	if(dummy_check) continue;

	cout << "Centering..." << endl;
	system->center();
      }

      // ******************************
      // center velocity
      // ******************************
      else if(arg=="centerv") {
	argcheck(argc, i, 0);
	i = i + 1;
	if(dummy_check) continue;

	cout << "Centering cm-velocity..." << endl;
	system->atypes.setmasses();
	system->centerv();
      }

      // ******************************
      // SRO
      // ******************************
      else if(arg=="sro") {
	double t1, t2, t3;
	int i1, i2;

	argcheck(argc, i, 5);
        t1 = atof(argv[i+1]);  // a
        i1 = system->atypes.typenumber(argv[i+2]);  // atom type
        i2 = system->atypes.typenumber(argv[i+3]);  // neighbour type
        t2 = atof(argv[i+4]);  // zmin
        t3 = atof(argv[i+5]);  // zmax
        i = i + 6;
	if(dummy_check) continue;

        t1 = (t1/sqrt(2) + t1)/2;

        system->init_nlist(t1, t1+0.01);
        system->update_symmnlist();
        cout << endl << endl;
        cout << "System SRO: " << system->sro(t1, i1, i2, t2, t3) << endl;
      }

      // ******************************
      // Smart SRO
      // ******************************
      else if(arg=="ssro") {
	double t1;
	int i1, i2;

	argcheck(argc, i, 3);
        t1 = atof(argv[i+1]);  // a
        i1 = system->atypes.typenumber(argv[i+2]);  // atom type
        i2 = system->atypes.typenumber(argv[i+3]);  // neighbour type
        i = i + 4;
	if(dummy_check) continue;

        t1 = (t1/sqrt(2) + t1)/2;

        system->init_nlist(t1, t1+0.01);
        system->update_symmnlist();
        cout << endl << endl;
	cout << "Smart short-range order parameter for the system: (time ssro)" << endl;
	cout << "sSRO  " << time << " " << system->smartsro(t1, i1, i2) << endl;
      }

      // ******************************
      // Local SRO
      // ******************************
      else if(arg=="lsro") {
	double t1;
	int i1, i2;

	argcheck(argc, i, 3);
        t1 = atof(argv[i+1]);  // a
        i1 = system->atypes.typenumber(argv[i+2]);  // atom type
        i2 = system->atypes.typenumber(argv[i+3]);  // neighbour type
        i = i + 4;
	if(dummy_check) continue;

        t1 = (t1/sqrt(2) + t1)/2;

        system->init_nlist(t1, t1+0.01);
        system->update_symmnlist();
	system->localsro(i1, i2, 0.0, t1);
      }

      // ******************************
      // Sfactor
      // ******************************
      else if(arg=="sfactor") {
	valarray<int> vi1;

	argcheck(argc, i, 4);
        system->set_lattice_constant(atof(argv[i+1]));
        vi1.resize(3);
        vi1[0] = atoi(argv[i+2]);  // nx
        vi1[1] = atoi(argv[i+3]);  // nx
        vi1[2] = atoi(argv[i+4]);  // nx
        i = i + 5;
	if(dummy_check) continue;

	cout << "Outputting time, |S|^2" << endl;
        cout << "|S|^2_(" << vi1[0] << "," << vi1[1] << "," << vi1[2] << "): " << time << " " << system->sfactor(vi1) << endl;
      }

      // ******************************
      // Sfactor with zlim
      // ******************************
      else if(arg=="sfactorz") {
	valarray<int> vi1;
	double t1;

	argcheck(argc, i, 5);
        system->set_lattice_constant(atof(argv[i+1]));
        t1 = atof(argv[i+2]);  // zlimit
        vi1.resize(3);
        vi1[0] = atoi(argv[i+3]);  // nx
        vi1[1] = atoi(argv[i+4]);  // nx
        vi1[2] = atoi(argv[i+5]);  // nx
        i = i + 6;
	if(dummy_check) continue;

	cout << "Outputting time, normalized |S|^2" << endl;
        cout << "|S|^2_(" << vi1[0] << "," << vi1[1] << "," << vi1[2] << "): " << time << " " << system->sfactor_znorm(t1, vi1) << endl;
      }

      // ******************************
      // Get T and other stuff
      // ******************************
      else if(arg=="temperature") {
	double t5, t1;

	argcheck(argc, i, 0);
	i = i + 1;
	if(dummy_check) continue;

	system->atypes.setmasses();
	system->updateT();
        t5 = system->rgyr();
 
        cout << endl;
        cout << "atoms:         " << system->atoms() << endl;
	cout << "T:             " << system->T << " K" << endl;
        cout << "R_g:           " << t5 << " Å" << endl;
        t1 = system->cmekin();
        cout << "E_{k,CM}:      " << t1 << " eV" << endl;
        cout << "E_{k,CM}/at.:  " << t1/system->atoms() << " eV" << endl;
      }

      // ******************************
      // Average T within sphere
      // ******************************
      else if(arg=="spheret") {
	vector<double> tempk;
	int startnei, at;
	double t1;
	int i1, i2;

	argcheck(argc, i, 1);
	t1 = atof(argv[i+1]);  // cutoff for averaging
	i = i + 2;
	if(dummy_check) continue;

	cout << "spheret: Replacing kinetic energy of each atom with value averaged over" << endl;
	cout << "         atoms within " << t1 << " A." << endl;

	// setup
	system->atypes.setmasses();
	system->updateT();
        system->init_nlist(t1,t1+0.0001);
        system->update_symmnlist();
	tempk.resize(system->atoms());
	for(i1=0;i1<tempk.size();i1++) tempk[i] = 0.0;

	// loop over atoms
	for(i1=0;i1<system->atoms();i1++) {
	  tempk[i1] += system->K[i1];
	  // loop over neighbours
	  for(i2=1;i2<=system->symmnlist[i1][0];i2++) {
	    tempk[i1] += system->K[system->symmnlist[i1][i2]];
	  }
	  tempk[i1] = tempk[i1]/(system->symmnlist[i1][0]+1);
	}

	// replace original K
	for(i1=0;i1<system->atoms();i1++) {
	  system->K[i1] = tempk[i1];
	}
      }

      // ******************************
      // Masses
      // ******************************
      else if(arg=="masses") {
	valarray<double> vd1;
	double t1;
	int i1, i2;

	argcheck(argc, i, 0);
	i = i + 1;
	if(dummy_check) continue;

	system->atypes.setmasses();

	vd1.resize(system->atypes.maxtypes);
	for(i1=0;i1<system->atypes.ntypes;i1++) vd1[i1] = 0.0;

	// mass per atom type
	for(i1=0;i1<system->atoms();i1++) {
	  vd1[system->atype[i1]] += system->atypes.mass[system->atype[i1]];
	}
	// total mass
	t1 = vd1.sum();

	
	cout << "Masses:" << endl;
	cout << "  Total mass of system: " << t1 << " amu" << endl;
	cout << "  Masses per atom type:" << endl;
	for(i1=0;i1<system->atypes.ntypes;i1++) {
	  i2 = system->atypes.types[i1];
	  cout << "    " << system->atypes.atypename[i2] << "   " << vd1[i2] << endl;
	}
	cout << "" << endl;
      }

      // ******************************
      // E_kin and velocity of center of mass
      // ******************************
      else if(arg=="cmekin") {
	int i1, i2;
	double t1, t2, t3;
	valarray<double> vd1;

	argcheck(argc, i, 0);
	i = i + 1;
	if(dummy_check) continue;

	system->atypes.setmasses();
 
        cout << endl;
        cout << "Outputting time, E_{k,CM} [eV], E_{k,CM}/atom [eV], vel [Å/fs], velocity (x,y,z) [Å/fs]" << endl;

        i1 = system->atoms();
        t1 = system->cmekin();
        t2 = t1/(double)i1;
	vd1.resize(system->dimension());
	vd1 = system->cmvel();
	t3 = 0;
	for(i2=0;i2<system->dimension();i2++) {
	  t3 += vd1[i2]*vd1[i2];
	}
	t3 = sqrt(t3);

        cout << "cmekin " << time << " " << t1 << " " << t2 << " " << t3 << " ";
	for(i2=0;i2<system->dimension();i2++) {
	  cout << vd1[i2] << " ";
	}
        cout << endl;
      }

      // ******************************
      // Extremal coordinates
      // ******************************
      else if(arg=="limits") {
	argcheck(argc, i, 0);
	i = i + 1;
	if(dummy_check) continue;

	cout << "Minimal and maximal atom coordinates:" << endl;
	cout << "limits x " << system->extrcoord(0, 0) << " " << system->extrcoord(0, 1) << endl;
	cout << "limits y " << system->extrcoord(1, 0) << " " << system->extrcoord(1, 1) << endl;
	cout << "limits z " << system->extrcoord(2, 0) << " " << system->extrcoord(2, 1) << endl;
        cout << endl;
      }

      // ******************************
      // Pair correlation function
      // ******************************
      else if(arg=="pair") {
	double t1, t2, t3, t4;

	argcheck(argc, i, 3);
        t1 = atof(argv[i+1]);  // a
        t2 = atof(argv[i+2]);  // b
        t3 = atof(argv[i+3]);  // dr
        i = i + 4;
	if(dummy_check) continue;

        cout << "Pair correlation function: (all lines start with pair)" << endl;
        cout << "------------------------------------------------------" << endl;

        t4 = t1;  // r
        while(t4 <= t2) {
          cout << "pair  " << t4 << "  " << system->paircorr(t4, t3) << endl;
          t4 += t3;
        }

        cout << "------------------------------------------------------" << endl;
        cout << endl;
      }

      // ******************************
      // q-correlation for all pairs
      // ******************************
      // uses: t1, i1
      else if(arg=="qcorr") {
	double t1;
	int i1, j, k;

	argcheck(argc, i, 2);
	t1 = atof(argv[i+1]);  // neighbour cutoff
	i1 = atoi(argv[i+2]);  // l
        i = i + 3;
	if(dummy_check) continue;

        system->init_nlist(t1,t1+0.0001);
        system->update_symmnlist();

        cout << "Outputting q-correlation numbers for all pairs. (i j c_ij)" << endl;

	// loop over atoms
	for(j=0;j<system->atoms();j++) {
	  // loop over neighbours
	  for(k=1;k<=system->symmnlist[j][0];k++) {
	    if(j>=system->symmnlist[j][k]) {
	      continue;
	    }
	    cout << "cij " << j << " " << system->symmnlist[j][k] << " " << system->q_correlation(j, system->symmnlist[j][k], i1) << endl;
	  }  // end of loop over neighbours
	}  // end of loop over atoms
      }

      // ******************************
      // Solid/liq. atoms, q-corr.
      // ******************************
      else if(arg=="qsolid") {
	double t1, t2, t3, t4;
	int i1, i2, i3, i4, i5, i6;
	int j, k, l;
	gsl_complex c;  // temp used by qsolid
	gsl_matrix_complex *qlm;  // data storage for qsolid

	argcheck(argc, i, 4);
	t1 = atof(argv[i+1]);  // neighbour cutoff
        t2 = atof(argv[i+2]);  // threshold for connected atoms
	t3 = atof(argv[i+3]);  // fraction of connected required for solid
	i3 = atoi(argv[i+4]);  // l
        i = i + 5;
	if(dummy_check) continue;

        cout << "Identifying solid/liquid atoms based on q-correlation. In V[i], 1 = solid, 0 = liquid." << endl;

	// check input
	if(t2 < 0 || t3 < 0) {
	  cerr << "qsolid: threshold and fraction have to be > 0, now: " << t2 << " " << t3 << endl;
	  exit(-1);
	}

        system->init_nlist(t1,t1+0.0001);
        system->update_symmnlist();

	// calculate q_lm for each atom and m
	qlm = gsl_matrix_complex_alloc(system->atoms(), 2*i3+1);
	for(j=0;j<system->atoms();j++) {
	  for(k=-i3;k<=i3;k++) {
	    gsl_matrix_complex_set(qlm, j, k+i3, system->q_lm(j, i3, k));
	  }
	}

	// loop over atoms
	i4 = 0;  // solid atoms
	i5 = 0;  // liquid atoms
	i6 = 0;  // undefined
	for(j=0;j<system->atoms();j++) {
	  i1 = 0;  // connected neighbours
	  i2 = 0;  // unconnected neighbours

	  // loop over neighbours, count connected and unconnected neighbours
	  for(k=1;k<=system->symmnlist[j][0];k++) {
	    /*
	      double Atom_System::q_correlation(int i, int j, int l) {

	      int m;  // loop
	      gsl_complex c;  // result

	      c = gsl_complex_rect(0, 0);
	      for(m=-l;m<=l;m++) {
	      c = gsl_complex_add(c, gsl_complex_mul(q_lm(i, l, m), gsl_complex_conjugate(q_lm(j, l, m))));
	      }  // end of loop over m
	      return gsl_complex_abs(c);
	    */

	    c = gsl_complex_rect(0, 0);
	    for(l=-i3;l<=i3;l++) {
	      c = gsl_complex_add(c, gsl_complex_mul(gsl_matrix_complex_get(qlm, j, l+i3), gsl_complex_conjugate(gsl_matrix_complex_get(qlm, system->symmnlist[j][k], l+i3))));
	    }
	    t4 = gsl_complex_abs(c);

	    if(t4 > t2) i1++;
	    else i2++;
	  }  // end of loop over neighbours

	  // determine if atom is solid or liquid
	  // - undefined
	  if( i2==0 && i1==0 ) {
	    system->V[j] = -1;
	    i6++;
	  }
	  // - solid
	  else if( i2==0 ) {
	    system->V[j] = 1;
	    i4++;
	  }
	  else if( (double)i1/(double)i2 > t3 ) {
	    system->V[j] = 1;
	    i4++;
	  }
	  // - liquid
	  else {
	    system->V[j] = 0;
	    i5++;
	  }
	}  // end of loop over atoms

	// free data
	gsl_matrix_complex_free(qlm);

	// report
	cout << "qsolid: Outputting: time, solid atoms, liquid atoms, lonely atoms" << endl;
	cout << "qsolid " << time << " " << i4 << " " << i5 << " " << i6 << endl;

	// check
	if(system->atoms() != i4+i5+i6) {
	  cerr << "qsolid: Error (bug): identified atoms don't sum up to n_atoms = " << system->atoms() << endl;
	  exit(-1);
	}
      }

      // ******************************
      // Hydrogen bond analysis
      // ******************************
      else if(arg=="hbond") {
	double cut13, cut12_23;
	double maxcut;
	double amin, amax;
	int i1, i2, i3;
	H_Bond hbond;

	argcheck(argc, i, 7);
        cut13 = atof(argv[i+1]);  // cutoff for r_13
        cut12_23 = atof(argv[i+2]);  // cutoff for r_12 and r_23
        amin = atof(argv[i+3])*M_PI/180.0;  // min angle
        amax = atof(argv[i+4])*M_PI/180.0;  // max angle
        i1 = atoi(argv[i+5]);  // type 1
        i2 = atoi(argv[i+6]);  // type 2 (angles of type t1-t2-t3 or t3-t2-t1)
        i3 = atoi(argv[i+7]);  // type 3
        i = i + 8;
	if(dummy_check) continue;

	// get max cutoff and init neighbour lists
	maxcut = cut13;
	if(cut12_23 > maxcut) maxcut = cut12_23;
        system->init_nlist(maxcut,maxcut+0.001);
        system->update_symmnlist();

	// analyze
	hbond.analyze(*system, amin, amax, cut13, cut12_23, i1, i2, i3, time);
      }

      // ******************************
      // Common neighbour analysis
      // ******************************
      else if(arg=="cna") {
	CNA cna;
	double t1;
	bool tb1;

	argcheck(argc, i, 2);
	cna.reset();
        t1 = atof(argv[i+1]);  // cutoff
	// for negative lattice constant, print unknown signatures
	tb1 = false;
	if(t1 < 0) {
	  tb1 = true;
	  cna.track_unknowns = true;
	  t1 = -t1;
	}
        cna.mode = atoi(argv[i+2]);  // mode
        i = i + 3;
	if(dummy_check) continue;

        cout << "Doing common neighbour analysis, cna-number will be in V[i]." << endl;

        cna.bondcut = t1;  // cutoff

	if(cna.mode==0) {
	  system->init_nlist(2*cna.bondcut+0.0001,2*cna.bondcut+0.0002);
	}
	else {
	  system->init_nlist(cna.bondcut+0.0001,cna.bondcut+0.0002);
	}
	system->update_symmnlist();

	cna.readsigs("cna_signatures.input");
	cna.calculate_and_analyze(*system, time);
	if(tb1) cna.savesigs("cna_signatures.output");
	cna.print_stats(time);
      }

      // ******************************
      // Coordination
      // ******************************
      else if(arg=="coordination") {
	double t1;
	int i1, i2;
	valarray<int> vi1;

	argcheck(argc, i, 1);
        t1 = atof(argv[i+1]);  // cutoff
        i = i + 2;
	if(dummy_check) continue;

	// calculate number of neighbours and get statistics
        cout << "Calculating the number of neighbours for each atom and placing in V[i]." << endl;

        system->init_nlist(t1,t1+0.0001);
        system->update_symmnlist();

	i2 = 1000;  // maximum coordination, hardcoded
	vi1.resize(i2);  // statistics
	for(i1=0;i1<i2;i1++) vi1[i1] = 0;  // init to zero

	// get coordination and statistics
	for(i1=0;i1<system->atoms();i1++) {
	  system->V[i1] = (double)system->symmnlist[i1][0];

	  if(system->symmnlist[i1][0] > i2-1) {
	    cerr << "coordination: Increase the (hardcoded) maximum number of neighbours. Now " << i2 << endl;
	    exit(-1);
	  }
	  vi1[system->symmnlist[i1][0]]++;
	}

	// output statistics
	cout << "Outputting statistics for coordination number: (n, number of atoms with n neighbours)" << endl;
        cout << "------------" << endl;
	for(i1=0;i1<i2;i1++) {
	  if(vi1[i1] != 0) {
	    cout << "coordstat " << i1 << " " << vi1[i1] << endl;
	  }
	}
        cout << "------------" << endl;
      }

      // ******************************
      // Lammps bonds
      // ******************************
      else if(arg=="lmp_bonds") {
	// input variables
	int id;
	string at1;
	string at2;
	double cut;
	string fname = "lmp_bonds.dat";
	// helpers
	ofstream file;
	int startnei;
	int i1, i2, i3;

	argcheck(argc, i, 4);
	id = atoi(argv[i+1]);             // bond id
	at1 = argv[i+2];                  // atom type 1 (name)
	at2 = argv[i+3];                  // atom type 2 (name)
	cut = atof(argv[i+4]);            // bond cutoff
	i = i + 5;
	if(dummy_check) continue;

	// --- start

	// init neighbour lists
        system->init_nlist(cut,cut+0.0001);

	// open file and output header if necessary
	cout << "Outputting bonds in LAMMPS format to file " << fname << "..." << endl;
	cout << "  atom types:  " << at1 << " " << at2 << endl;
	cout << "  cutoff:      " << cut << endl;
	cout << "  bond id:     " << id << endl;
	if(lmp_bonds==-1) file.open(fname.c_str(), ios::trunc);
	else file.open(fname.c_str(), ios::app);
	assert(file);

	// output header if necessary
	if(lmp_bonds==-1) {
	  lmp_bonds++;
	  file << "" << endl;
	  file << "Bonds" << endl;
	  file << "" << endl;
	}

	// count bond types
	lmp_bond_types++;

	// loop over atoms
	startnei = 0;
	for(i1=0;i1<system->atoms();i1++) {
	  // loop over neighbour list
	  for(i2=startnei+1;i2<=startnei + system->nlist[startnei];i2++) {
	    i3 = system->nlist[i2];  // other atom

	    // check for bonds
	    if( (system->atype[i1]==system->atypes.typenumber(at1) && system->atype[i3]==system->atypes.typenumber(at2)) || (system->atype[i1]==system->atypes.typenumber(at2) && system->atype[i3]==system->atypes.typenumber(at1)) ) {
	      if(system->dr(i1, i3) < cut) {
		// bond found
		lmp_bonds++;
		if(i1 < i3) {
		  file << lmp_bonds << " " << id << " " << i1+1 << " " << i3+1 << endl;
		}
		else {
		  file << lmp_bonds << " " << id << " " << i3+1 << " " << i1+1 << endl;
		}
	      }
	    }
	  }
	  startnei += system->nlist[startnei] + 1;
	}

	file.close();
      }

      // ******************************
      // Lammps angles
      // ******************************
      else if(arg=="lmp_angles") {
	// input variables
	int id;
	string at1;
	string at2;
	string at3;
	double cut12;
	double cut23;
	string fname = "lmp_angles.dat";
	// helpers
	ofstream file;
	int n1, n2, n3;  // atoms
	int i1, i2, i3;
	int t1, t2, t3;  // types (integers) of at1, at2, at3

	argcheck(argc, i, 6);
	id = atoi(argv[i+1]);             // bond id
	at1 = argv[i+2];                  // atom type 1 (name)
	at2 = argv[i+3];                  // atom type 2 (name)
	at3 = argv[i+4];                  // atom type 3 (name)
	cut12 = atof(argv[i+5]);            // bond cutoff 1-2
	cut23 = atof(argv[i+6]);            // bond cutoff 2-3
	i = i + 7;
	if(dummy_check) continue;

	// --- start

	// init neighbour lists
	if(cut12 > cut23) {
	  system->init_nlist(cut12,cut12+0.0001);
	}
	else {
	  system->init_nlist(cut23,cut23+0.0001);
	}
        system->update_symmnlist();

	// type numbers
	t1 = system->atypes.typenumber(at1);
	t2 = system->atypes.typenumber(at2);
	t3 = system->atypes.typenumber(at3);

	// open file and output header if necessary
	cout << "Outputting angles in LAMMPS format to file " << fname << "..." << endl;
	cout << "  atom types:  " << at1 << " (" << t1 << ") -- " << at2 << " (" << t2 << ") -- " << at3 << " (" << t3 << ")" << endl;
	cout << "  cutoff 1-2:  " << cut12 << endl;
	cout << "  cutoff 2-3:  " << cut23 << endl;
	cout << "  angle id:    " << id << endl;
	if(lmp_angles==-1) file.open(fname.c_str(), ios::trunc);
	else file.open(fname.c_str(), ios::app);
	assert(file);

	// output header if necessary
	if(lmp_angles==-1) {
	  lmp_angles++;
	  file << "" << endl;
	  file << "Angles" << endl;
	  file << "" << endl;
	}

	// count angle types
	lmp_angle_types++;

	// loop over atoms
	for(i1=0;i1<system->atoms();i1++) {
	  n1 = i1;
	  // loop over neighbours
	  for(i2=1;i2<=system->symmnlist[n1][0];i2++) {
	    n2 = system->symmnlist[n1][i2];

	    // check type and distance
	    if( (system->atype[n1]==t1 || system->atype[n1]==t3) && system->atype[n2]==t2 ) {
	      if(system->dr(n1, n2) < cut12) {

		// loop over neighbours of atom n2
		for(i3=1;i3<=system->symmnlist[n2][0];i3++) {
		  n3 = system->symmnlist[n2][i3];

		  // avoid loopback
		  if(n3 <= n1) continue;

		  // check type and distance
		  if( (system->atype[n1]==t1 && system->atype[n3]==t3) || (system->atype[n1]==t3 && system->atype[n3]==t1) ) {
		    if(system->dr(n2, n3) < cut23) {

		      // warning
		      if(system->dr(n1,n3) < cut12 || system->dr(n1,n3) < cut23) {
			cout << "lmp_angles: Warning: In angle " << n1+1 << "-" << n2+1 << "-" << n3+1 << ", " << n1+1 << " and " << n3+1 << " are also within cutoff." << endl;
		      }

		      // add angle
		      lmp_angles++;
		      file << lmp_angles << " " << id << " " << n1+1 << " " << n2+1 << " " << n3+1 << endl;
		    }
		  }

		}  // end of loop over neighbours of atom n2
	      }
	    }
	  }  // end of loop over neighbours
	}  // end of loop over atoms

	file.close();
      }

      // ******************************
      // Lammps dihedrals
      // ******************************
      else if(arg=="lmp_dihedrals") {
	// input variables
	int id;
	string at1;
	string at2;
	string at3;
	string at4;
	double cut12, cut23, cut34;
	string fname = "lmp_dihedrals.dat";
	// helpers
	ofstream file;
	int n1, n2, n3, n4;  // atoms
	int t1, t2, t3, t4;  // types corresp. to at1-4
	int i1, i2, i3, i4;

	argcheck(argc, i, 8);
	id = atoi(argv[i+1]);             // bond id
	at1 = argv[i+2];                  // atom type 1 (name)
	at2 = argv[i+3];                  // atom type 2 (name)
	at3 = argv[i+4];                  // atom type 3 (name)
	at4 = argv[i+5];                  // atom type 3 (name)
	cut12 = atof(argv[i+6]);          // bond cutoff
	cut23 = atof(argv[i+7]);          // bond cutoff
	cut34 = atof(argv[i+8]);          // bond cutoff
	i = i + 9;
	if(dummy_check) continue;

	// --- start

	// init neighbour lists
	{
	  double cut;
	  cut = cut12;
	  if(cut23 > cut) cut = cut23;
	  if(cut34 > cut) cut = cut34;
	  system->init_nlist(cut,cut+0.0001);
	  system->update_symmnlist();
	}

	// open file and output header if necessary
	cout << "Outputting dihedrals in LAMMPS format to file " << fname << "..." << endl;
	cout << "  atom types:  " << at1 << " (" << t1 << ") -- " << at2 << " (" << t2 << ") -- " << at3 << " (" << t3 << ") -- " << at4 << " (" << t4 << ")" <<  endl;
	cout << "  cutoff 1-2:  " << cut12 << endl;
	cout << "  cutoff 2-3:  " << cut23 << endl;
	cout << "  cutoff 3-4:  " << cut34 << endl;
	cout << "  dihedral id: " << id << endl;
	if(lmp_dihedrals==-1) file.open(fname.c_str(), ios::trunc);
	else file.open(fname.c_str(), ios::app);
	assert(file);

	// output header if necessary
	if(lmp_dihedrals==-1) {
	  lmp_dihedrals++;
	  file << "" << endl;
	  file << "Dihedrals" << endl;
	  file << "" << endl;
	}

	// count dihedral types
	lmp_dihedral_types++;

	// The algorithm below is now a bit slower than optimal with
	// respect to using the neighbour lists, but easier to program.

	t1 = system->atypes.typenumber(at1);
	t2 = system->atypes.typenumber(at2);
	t3 = system->atypes.typenumber(at3);
	t4 = system->atypes.typenumber(at4);

	// loop over atoms (n1)
	for(i1=0;i1<system->atoms();i1++) {
	  n1 = i1;
	  // loop over neighbours of n1
	  for(i2=1;i2<=system->symmnlist[n1][0];i2++) {
	    n2 = system->symmnlist[n1][i2];

	    // check type and distance
	    if( (system->atype[n1]==t1 && system->atype[n2]==t2) || (system->atype[n1]==t4 && system->atype[n2]==t3) ) {
	      if(system->dr(n1, n2) < cut12) {

		// loop over neighbours of n2
		for(i3=1;i3<=system->symmnlist[n2][0];i3++) {
		  n3 = system->symmnlist[n2][i3];

		  // avoid loopback
		  if(n3==n1) continue;

		  // check type and distance
		  if( (system->atype[n2]==t2 && system->atype[n3]==t3) || (system->atype[n2]==t3 && system->atype[n3]==t2) ) {
		    if(system->dr(n2, n3) < cut23) {

		      // loop over neighbours of n3
		      for(i4=1;i4<=system->symmnlist[n3][0];i4++) {
			n4 = system->symmnlist[n3][i4];

			// avoid loopback
			if(n4==n2) continue;

			// check type and distance
			if( (system->atype[n3]==t3 && system->atype[n4]==t4) || (system->atype[n3]==t2 && system->atype[n4]==t1) ) {
			  if(system->dr(n3, n4) < cut34) {

			    // dihedral found, avoid duplicates
			    if(n4 < n1) continue;

			    // checks and warnings
			    if(n1==n2 || n1==n3 || n1==n4 || n2==n3 || n2==n4 || n3==n4) {
			      cerr << "lmp_angles: Bug: Duplicate atoms in dihedral: " << n1+1 << " " << n2+1 << " " << n3+1 << " " << n4+1 << endl;
			      exit(-1);
			    }
			    /*
			    if(system->dr(n1,n3) < cut) {
			      cout << "lmp_angles: Warning: In dihedral " << n1+1 << "-" << n2+1 << "-" << n3+1 << " " << n4+1 << ", " << n1+1 << " and " << n3+1 << " are also within cutoff!" << endl;
			    }
			    if(system->dr(n1,n4) < cut) {
			      cout << "lmp_angles: Warning: In dihedral " << n1+1 << "-" << n2+1 << "-" << n3+1 << " " << n4+1 << ", " << n1+1 << " and " << n4+1 << " are also within cutoff!" << endl;
			    }
			    if(system->dr(n2,n4) < cut) {
			      cout << "lmp_angles: Warning: In dihedral " << n1+1 << "-" << n2+1 << "-" << n3+1 << " " << n4+1 << ", " << n2+1 << " and " << n4+1 << " are also within cutoff!" << endl;
			    }
			    */

			    // add dihedral
			    lmp_dihedrals++;
			    file << lmp_dihedrals << " " << id << " " << n1+1 << " " << n2+1 << " " << n3+1 << " " << n4+1 << endl;
			  }
			}
		      }  // end of loop over neighbours of n3
		    }
		  }
		}  // end of loop over neighbours of atom n2
	      }
	    }
	  }  // end of loop over neighbours
	}  // end of loop over atoms

	file.close();
      }

      // ******************************
      // Bond defects
      // ******************************
      else if(arg=="bdef") {
	int i1, i2, i3, i4, i5, i6, i7, i8;
	bool tb1;
	vector< vector<double> > md1;
	vector< vector<int> > mi1, mi2, mi3;
	double t1;

	argcheck(argc, i, 1);
	i5 = atoi(argv[i+1]);  // number of specified atom types
	i++;
	if(i5 <=0) {
	  cerr << "You have to have at least 1 specified atom type in bdef." << endl;
	  exit(-1);
	}
	/*
	  For example, for polyethylene with C (1) and H (2), each H having
	  one C neighbour and each C two C and two H neighbours. The first column
	  needs (and is) only specified in mi1.

	  For this example the input line would be:
	    bdef 2 2 C H 1.5 2 C 2.0 2 2 H H 1.5 0 C 1.5 1

	  mi1:           which means:
	  1 1 2          C C H
          2 1            H C

	  mi2:           which means:
	  1 2 2          C 2C 2H
          2 1            H 1C
	*/

	mi1.resize(i5);  // neighbour types
	md1.resize(i5);  // neighbour cutoffs
	mi2.resize(i5);  // required number of neighbours
	mi3.resize(i5);  // number of neighbours

	// default to non-defected atom
	for(i3=0;i3<system->atoms();i3++) {
	  system->V[i3] = 0;
	}

	// loop over specified atom types
	t1 = 0;  // maximum cutoff
	for(i3=0;i3<i5;i3++) {
	  i4 = atoi(argv[i+1]);  // number of neighbour constraints for atom type
	  i++;
	  mi1[i3].resize(i4+1);
	  mi2[i3].resize(i4+1);
	  mi3[i3].resize(i4+1);
	  md1[i3].resize(i4+1);
	  // reset required numbers
	  for(i6=0;i6<mi2[i3].size();i6++) {
	    mi2[i3][i6] = 0;
	  }
	  i6 = system->atypes.typenumber(argv[i+1]);  // atom type for which the constraints are
	  i++;
	  mi1[i3][0] = i6;
	  // loop over neighbours
	  for(i6=0;i6<i4;i6++) {
	    mi1[i3][1+i6] = system->atypes.typenumber(argv[i+1]);  // neighbour type
	    i++;
	    md1[i3][1+i6] = atof(argv[i+1]);  // neighbour cutoff
	    i++;
	    if(md1[i3][1+i6] > t1) t1 = md1[i3][1+i6];  // find maximum cutoff
	    mi2[i3][1+i6] = atoi(argv[i+1]);  // required number of neighbours
	    i++;
	  }
	}
	i++;

	if(dummy_check) continue;

	// --- done taking input

	// report
	cout << endl;
	cout << "Starting bond defect analysis" << endl;
	for(i3=0;i3<i5;i3++) {
	  cout << "For atom type " << system->atypes.name(mi1[i3][0]) << " (" << mi1[i3][0] << ") requiring neighbours:" << endl;
	  for(i4=1;i4<mi1[i3].size();i4++) {
	    cout << "              " << mi2[i3][i4] << " " << system->atypes.name(mi1[i3][i4]) << " closer than " << md1[i3][i4] << endl;
	  }
	}

	// init neighbour lists
        system->init_nlist(t1,t1+0.0001);
        system->update_symmnlist();

	// --- loop over atoms
	for(i3=0;i3<system->atoms();i3++) {

	  tb1 = false;  // are their constraints for atoms of this type?

	  // find out index of this atom type
	  for(i4=0;i4<mi1.size();i4++) {
	    if(system->atype[i3]==mi1[i4][0]) {
	      i8 = i4;  // i8 = first index of m** for this atom
	      tb1 = true;
	      break;
	    }
	  }

	  if(!tb1) {
	    continue;
	  }

	  // reset numbers of neighbours (but only the required elements)
	  mi3 = mi2;
	  for(i7=0;i7<mi3[i8].size();i7++) {
	    mi3[i8][i7] = 0;
	  }
	  // loop over neighbours
	  for(i4=1;i4<=system->symmnlist[i3][0];i4++) {
	    // loop over specified neighbour types
	    for(i7=1;i7<mi1[i8].size();i7++) {
	      // check if the neighbour is of this type
	      if(system->atype[system->symmnlist[i3][i4]]==mi1[i8][i7]) {
		if(system->dr(i3,system->symmnlist[i3][i4]) < md1[i8][i7]) {
		  mi3[i8][i7]++;  // calculate number of neighbours of this type
		}
	      }
	    }  // end of loop over neighbour types
	  }  // end of loop over neighbours

	  // --- check if this atom matches the description

	  if(mi3==mi2) {
	    system->V[i3] = 0;
	  }
	  else {
	    system->V[i3] = 1;

	    /*
	    for(i7=0;i7<i5;i7++) {
	      cout << "For atom " << i3 << " " << system->atypes.name(system->atype[i3]) << endl;
	      for(i4=1;i4<mi1[i7].size();i4++) {
		cout << "              " << mi3[i7][i4] << " " << system->atypes.name(mi1[i7][i4]) << " closer than " << md1[i7][i4] << endl;
	      }
	    }
	    */
	  }

	}  // end of loop over atoms
      }

      // ******************************
      // Identify molecules
      // ******************************
      else if(arg=="mdef") {
	int i1, i2, i3, i4, i5, i6, i7, i8;
	bool tb1, tb2;
	vector< vector<double> > md1;
	vector< vector<int> > mi1, mi2, mi3;
	valarray<int> vi1, vi2;
	valarray<double> vd1;
	double t1;

	argcheck(argc, i, 1);
	i5 = atoi(argv[i+1]);  // number of specified atom types
	i++;
	if(i5 <=0) {
	  cerr << "You have to have at least 1 specified atom type in bdef." << endl;
	  exit(-1);
	}
	mi1.resize(i5);  // neighbour types
	md1.resize(i5);  // neighbour cutoffs
	vi1.resize(i5);  // number of atoms of each type
	vd1.resize(i5);  // radia within which to consider neighbours

	bool debug = false;

	// default to atoms not in a molecule like this:
	for(i3=0;i3<system->atoms();i3++) {
	  system->V[i3] = 1;
	}

	/*
	  For example, for ethylene carbonate (EC), the input line is:
	    mdef 5   1 O 1 C 1.7   1 C 1.7 3 O 1.7 O 1.7 O 1.7   2 O 2 C 1.7 C 1.7   2 C 4 O 1.7 C 1.7 H 1.7 H 1.7   4 H 1 C 1.7
                 |                 | |  |  | |  |  |  |  |  |
                 |                 | |  |  | |  |  >  >  >  >                                 etc. etc.
                 |                 | |  |  | >  >                 of which one O within 1.7 A
                 |                 | |  >  > with 3 neighbours (disregarding anything farther than 1.7 A)
                 |                 | >   C              
                 |                 > one                  
                 > Five types of atoms               

	    Assuming atom types:
	      C  1
	      H  2
	      O  3

    	  mi1:           which means:
	  3 1            O: C
	  1 3 3 3        C: O O O
          3 1 1          O: C C
          1 3 1 2 2      C: O C H H
          2 1            H: C

	  vi1: (how many of each)
	  1 1 2 2 4

	  vd1: (ignore extra atoms outside this radius for this type)

	  vi2: copy of vi1 for counting
	  mi2: for counting neighbours of one atom

	*/

	// --- read input

	// loop over specified atom types
	t1 = 0;  // maximum cutoff
	for(i3=0;i3<i5;i3++) {
	  vi1[i3] = atoi(argv[i+1]);  // number of atoms of this type
	  i++;

	  i1 = system->atypes.typenumber(argv[i+1]);  // atom type for which the constraints are
	  i++;
	  vd1[i3] = atof(argv[i+1]);  // cutoff radius within which to consider neighbours
	  i++;
	  i4 = atoi(argv[i+1]);  // number of neighbour constraints for atom type
	  i++;
	  mi1[i3].resize(i4+1);
	  md1[i3].resize(i4+1);
	  mi1[i3][0] = i1;

	  // loop over neighbours
	  for(i6=1;i6<mi1[i3].size();i6++) {
	    mi1[i3][i6] = system->atypes.typenumber(argv[i+1]);  // neighbour type
	    i++;
	    md1[i3][i6] = atof(argv[i+1]);  // neighbour cutoff
	    i++;
	    if(md1[i3][i6] > t1) t1 = md1[i3][i6];  // find maximum cutoff
	  }
	}
	i++;

	if(dummy_check) continue;

	// --- done taking input

	// init neighbour lists
        system->init_nlist(t1,t1+0.0001);
        system->update_symmnlist();

	// --- report

	cout << endl;
	cout << "Identifying molecules. Requiring the following atoms:" << endl;
	for(i3=0;i3<mi1.size();i3++) {
	  cout << vi1[i3] << " atoms of type " << system->atypes.name(mi1[i3][0]) << " (" << mi1[i3][0] << ") with neighbours:" << endl;
	  for(i4=1;i4<mi1[i3].size();i4++) {
	    cout << "              " << system->atypes.name(mi1[i3][i4]) << " closer than " << md1[i3][i4] << endl;
	  }
	  cout << "              ignoring extra neighbours further away than " << vd1[i3] << " A." << endl;
	}

	// --- start

	// loop over blocks / molecules
	//   - At the moment, very inefficient.
	for(i1=0;i1<system->blocknat.size();i1++) {
	  // check that molecule has correct size
	  if(system->blocknat[i1] != vi1.sum()) continue;

	  // reset number of atoms of each type
	  vi2.resize(vi1.size());
	  for(i3=0;i3<vi2.size();i3++) {
	    vi2[i3] = vi1[i3];
	  }

	  // loop over atoms
	  for(i2=0;i2<system->atoms();i2++) {

	    // identify for atoms in this molecule
	    if(system->block[i2]==i1) {

	      if(debug) cout << "Atom " << i2 << " of type " << system->atypes.name(system->atype[i2]) << " (" << system->atype[i2] << ")" << endl;

	      // reset neighbour counter
	      mi2.resize(mi1.size());
	      for(i3=0;i3<mi2.size();i3++) {
		mi2[i3].resize(mi1[i3].size());
		for(i4=0;i4<mi2[i3].size();i4++) {
		  mi2[i3][i4] = 1;
		}
	      }

	      // loop over neighbours of this atom
	      for(i3=1;i3<=system->symmnlist[i2][0];i3++) {

		// loop over neighbour types of all relevant atom types
		for(i4=0;i4<mi1.size();i4++) {
		  if(system->atype[i2]==mi1[i4][0] && vi2[i4] > 0) {
		    // loop over requested neighbours
		    for(i5=1;i5<mi1[i4].size();i5++) {
		      if(system->atype[system->symmnlist[i2][i3]] == mi1[i4][i5]) {
			if(mi2[i4][i5]==1) {
			  if(system->dr(i2, system->symmnlist[i2][i3]) < md1[i4][i5]) {
			    mi2[i4][i5]--;
			    if(debug) cout << "    type " << i4 << ": has neighbour of type " << system->atypes.name(mi1[i4][i5]) << " (" << mi1[i4][i5] << ")" << endl;
			    break;
			  }
			}
		      }
		    }  // end of loop over requested neighbours
		  }
		}  // end of loop over neighbour types of all relevant atom types

	      } // end of loop over neighbours of this atom


	      // --- does the atom types match one of the requested?

	      i8 = -1;  // atom type this atom matches

	      // loop over atom types
	      for(i4=0;i4<mi1.size();i4++) {
		// only consider atom types still available
		tb2 = false;  // match?
		if(vi2[i4] > 0) {
		  if(debug) cout << "  ... type " << i4 << " still available..." << endl;
		  tb2 = true;  // match?
		  // loop over requested neighbours
		  for(i5=1;i5<mi1[i4].size();i5++) {
		    if(mi2[i4][i5]==1) {
		      tb2 = false;
		    }
		  }
		  if(debug) {
		    if(tb2) cout << "  ... all requested neighbours are there..." << endl;
		    else cout << "  ... all requested neighbours are not there..." << endl;
		  }

		  // if match, check that there are no extra neighbours
		  if(tb2) {
		    // count neighbours within specified cutoff
		    i7 = 0;
		    for(i6=1;i6<=system->symmnlist[i2][0];i6++) {
		      if(system->dr(i2, system->symmnlist[i2][i6]) < vd1[i4]) {
			i7++;
		      }
		    }

		    if(debug) cout << "  ... there are " << i7 << " neighbours within range..." << endl;

		    // check
		    if(i7 != mi1[i4].size()-1) {
		      tb2 = false;
		    }
		  }
		}

		if(tb2) {
		  i8 = i4;
		  break;
		}
	      }  // end of loop over atom types

	      // check if we can already say this molecule does not satisfy the conditions
	      if(i8 < 0) {
		tb1 = false;
		if(debug) cout << "  ... no match" << endl;
		break;
	      }
	      else {
		// this atom is ok
		tb1 = true;
		if(debug) cout << "  ... match: type " << i8 << endl;
		vi2[i8]--;
	      }

	    }
	  }  // end of loop over atoms

	  // set V=0 if match
	  if(tb1) {
	    for(i2=0;i2<system->atoms();i2++) {
	      // identify for atoms in this molecule
	      if(system->block[i2]==i1) {
		system->V[i2] = 0;
	      }
	    }
	  }

	}  // end of loop over blocks / molecules

      }

      // ******************************
      // Rotate
      // ******************************
      else if(arg=="rotate") {
	double phi, theta, psi;

	argcheck(argc, i, 3);
	phi = atof(argv[i+1]);
	theta = atof(argv[i+2]);
	psi = atof(argv[i+3]);
	i = i + 4;
	if(dummy_check) continue;

	cout << "Rotating with (phi,theta,psi) = ( " << phi << " " << theta << " " << psi << " )..." << endl;
	system->rotate_euler(phi, theta, psi);
      }

      // ******************************
      // Rotateaxis
      // ******************************
      else if(arg=="rotateaxis") {
	double phi, theta, delta;

	argcheck(argc, i, 3);
	phi = atof(argv[i+1]);
	theta = atof(argv[i+2]);
	delta = atof(argv[i+3]);
	i = i + 4;
	if(dummy_check) continue;

	cout << "Rotating with (phi,theta,delta) = ( " << phi << " " << theta << " " << delta << " )..." << endl;
	system->rotate_axis(phi, theta, delta);
      }

      // ******************************
      // Rotatevec
      // ******************************
      else if(arg=="rotatevec") {
	double phi, theta, delta;
	double x, y, z;
	double r;

	argcheck(argc, i, 4);
	x = atof(argv[i+1]);
	y = atof(argv[i+2]);
	z = atof(argv[i+3]);
	delta = atof(argv[i+4]);
	i = i + 5;
	if(dummy_check) continue;

	// figure out rotation in terms of angles
	r = sqrt(x*x + y*y + z*z);
	theta = acos(z/r);
	phi = atan(y/x);

	cout << "Rotating around axis (x,y,z) = ( " << x << " " << y << " " << z << " ) by angle " << delta << endl;
	cout << "  This corresponds to \"rotateaxis\" with (phi,theta,delta) = ( " << phi << " " << theta << " " << delta << " )" << endl;
	system->rotate_axis(phi, theta, delta);
      }

      // ******************************
      // Translate
      // ******************************
      else if(arg=="translate") {
	valarray<double> dr(3);

	argcheck(argc, i, 3);
	dr[0] = atof(argv[i+1]);
	dr[1] = atof(argv[i+2]);
	dr[2] = atof(argv[i+3]);
	i = i + 4;
	if(dummy_check) continue;

	cout << "Translating with ( " << dr[0] << " " << dr[1] << " " << dr[2] << " )..." << endl;
	system->translate(dr, true);
      }

      // ******************************
      // Move to positive coordinates
      // ******************************
      else if(arg=="movepos") {
	valarray<double> dr(3);

	argcheck(argc, i, 0);
	i = i + 1;
	if(dummy_check) continue;

        dr[0] = -system->extrcoord(0, 0);
        dr[1] = -system->extrcoord(1, 0);
        dr[2] = -system->extrcoord(2, 0);
	system->translate(dr, true);
      }

      // ******************************
      // Scale
      // ******************************
      else if(arg=="scale") {
	double t1;

	argcheck(argc, i, 1);
	t1 = atof(argv[i+1]);  // factor to scale by
	i = i + 2;
	if(dummy_check) continue;

	system->scalec(t1);
      }

      // ******************************
      // Unit of length
      // ******************************
      else if(arg=="unit_l") {
	double t1;

	argcheck(argc, i, 1);
	t1 = atof(argv[i+1]);  // factor to scale by
	i = i + 2;
	if(dummy_check) continue;

	system->unit_l(t1);
      }

      // ******************************
      // Unit of time
      // ******************************
      else if(arg=="unit_t") {
	double t1;

	argcheck(argc, i, 1);
	t1 = atof(argv[i+1]);  // factor to scale by
	i = i + 2;
	if(dummy_check) continue;

	system->unit_t(t1);
      }

      // ******************************
      // Rename atom type
      // ******************************
      else if(arg=="rename_type") {
	int i1;
	string t1;

	argcheck(argc, i, 2);
	i1 = atoi(argv[i+1]);  // atom type, integer
	t1 = argv[i+2];        // new name
	i = i + 3;
	if(dummy_check) continue;

	cout << "Renaming atom type " << i1 << " from \"" << system->atypes.atypename[i1] << "\" to \"" << t1 << "\"." << endl;
	system->atypes.setname(i1, t1);
      }

      // ******************************
      // Add velocity downwards
      // ******************************
      else if(arg=="addveldz") {
        double t1, t2;
	int i1, i2;

	argcheck(argc, i, 2);
	t1 = atof(argv[i+1]);  // vel. to add, Å/fs
	t2 = atof(argv[i+2]);  // to atoms above z=t2
	i = i + 3;
	if(dummy_check) continue;

	i1 = 0;  // number of atoms velocity added to
	for(i2=0;i2<system->atoms();i2++) {
	  if(system->x0[3*i2 + 2] > t2) {
	    system->x1[3*i2 + 2] += t1;
	    i1++;
	  }
	}
	cout << "Added velocity (0, 0, " << t1 << ") to " << i1 << " atoms above z = " << t2 << endl;
      }

      // ******************************
      // Add E_k
      // ******************************
      else if(arg=="addek") {
	double t1;

	argcheck(argc, i, 1);
	t1 = atof(argv[i+1]);  // E/at. to add
	i = i + 2;
	if(dummy_check) continue;

	system->atypes.setmasses();
	system->addEk(t1);
      }

      // ******************************
      // Move atom
      // ******************************
      else if(arg=="moveatom") {
	int i1;
	valarray<double> dr(3);

	argcheck(argc, i, 4);
        i1 = atoi(argv[i+1]);
	dr[0] = atof(argv[i+2]);
	dr[1] = atof(argv[i+3]);
	dr[2] = atof(argv[i+4]);
	i = i + 5;
	if(dummy_check) continue;

	cout << "Translating atom " << i1 << " with ( " << dr[0] << " " << dr[1] << " " << dr[2] << " )..." << endl;
	system->moveatom(i1, dr, true);
      }

      // ******************************
      // Get moved atoms
      // ******************************
      else if(arg=="getmoved") {
	double t1;
	string dummy;
	Atom_System *other;

	argcheck(argc, i, 2);
	t1 = atof(argv[i+1]);
        dummy = argv[i+2];
	i = i + 3;
	if(dummy_check) continue;

        // load other system
        other = new Atom_System();
	other->readxyz(dummy.c_str(), 0, 1);

	cout << "Comparing system to file " << dummy << " to get atoms moved by more than " << t1 << endl;
        system->getmoved(t1, *other);
        delete other;
        cout << "...done." << endl;
      }

      // ******************************
      // Get moved atoms
      // ******************************
      else if(arg=="getcenter") {
	valarray <double> center(3);  // geometrical center of system

	argcheck(argc, i, 0);
	i = i + 1;
	if(dummy_check) continue;

	cout << "Warning: getcenter: Trying to figure out geometrical center of atoms taking" << endl;
	cout << "                    periodics into account. Assuming that there is a relatively" << endl;
	cout << "                    small and compact cluster of atoms of which to get the" << endl;
	cout << "                    center of, so that this is well defined." << endl;


	// init
	for(int j=0; j<3; j++) {
	  center[j] = 0.0;
	}

	// get center
	for(int at=0; at<system->atoms(); at++) {
	  for(int j=0; j<3; j++) {
	    if(system->x0[3*at + j] < 0.0 && system->pbc[j]==1) center[j] += system->x0[3*at + j] + system->box[j];
	    else center[j] += system->x0[3*at + j];
	  }
	}
	// divide with number of atoms and wrap center within box
	for(int j=0; j<3; j++) {
	  center[j] /= system->atoms();
	  if(center[j] > system->box[j]/2.0) center[j] -= system->box[j];
	}

	cout << "getcenter: getcenter time center-x center-y center-z" << endl;
	cout << "getcenter " << time << " " << center[0] << " " << center[1] << " " << center[2] << endl;
      }

      // ******************************
      // Amplitude of r(t)
      // ******************************
      else if(arg=="art") {
	double t1;
	int i1;

	argcheck(argc, i, 0);
	i = i + 1;
	if(dummy_check) continue;

	// check
	if(frames <= 1) {
	  cerr << "Cannot get amplitude of r(t), number of frames: " << frames << endl;
	}
	// collect data
	else {
	  system->center();
	  if(rmin.size()==0) {
	    rmin.resize(system->atoms());
	    rmax.resize(system->atoms());
	    for(i1=0;i1<system->atoms();i1++) {
	      t1 = sqrt(system->x0[3*i1]*system->x0[3*i1] + system->x0[3*i1+1]*system->x0[3*i1+1] + system->x0[3*i1+2]*system->x0[3*i1+2]);
	      rmin[i1] = t1;
	      rmax[i1] = t1;
	    }
	  }
	  else {
	    if(rmin.size() != system->atoms()) {
	      cerr << "Don't use command \"art\" for movie-files with changing number of atoms! rmin.size(), atoms: " << rmin.size() << ", " << system->atoms() << endl;
	      exit(-1);
	    }
	    for(i1=0;i1<system->atoms();i1++) {
	      t1 = sqrt(system->x0[3*i1]*system->x0[3*i1] + system->x0[3*i1+1]*system->x0[3*i1+1] + system->x0[3*i1+2]*system->x0[3*i1+2]);
	      if(rmin[i1] > t1) rmin[i1] = t1;
	      if(rmax[i1] < t1) rmax[i1] = t1;
	    }
	  }
	}
      }

      // ******************************
      // Get displacements relative to first frame (does not support going over periodics many times yet)
      // ******************************
      else if(arg=="disp") {
	argcheck(argc, i, 0);
	i = i + 1;
	if(dummy_check) continue;

	// check
	if(frames <= 1) {
	  cerr << "disp: Cannot get displacements relative to first frame as number of frames is " << frames << endl;
	}

	// init
	//
	// previous frame will be stored in prev_frame
	// displacement will be accumulated in prev_frame->dx0[]
	if(frame==1) {
	  prev_frame = *system;

	  for(int j=0; j<system->atoms(); j++) {
	    prev_frame.dx0[j] = 0.0;
	    system->V[j] = 0.0;
	  }
	}
	// get displacement and store this position for next frame
	else {
	  for(int j=0; j<system->atoms(); j++) {
	    system->V[j] = 0.0;
	    for(int k=0;k<3;k++) {
	      prev_frame.dx0[j*3+k] += system->x0[j*3+k] - prev_frame.x0[j*3+k];
	      prev_frame.x0[j*3+k] = system->x0[j*3+k];

	      // calculate total displacement to V
	      system->V[j] += prev_frame.dx0[j*3+k]*prev_frame.dx0[j*3+k];
	    }
	    system->V[j] = sqrt(system->V[j]);
	  }
	}
      }

      // ******************************
      // Get distance from (x,y,z)
      // ******************************
      else if(arg=="distfrom") {
	valarray<double> vd1(3);
	int i1;

	argcheck(argc, i, 3);
	vd1[0] = atof(argv[i+1]);
	vd1[1] = atof(argv[i+2]);
	vd1[2] = atof(argv[i+3]);
	i = i + 4;
	if(dummy_check) continue;

	cout << "Outputting time, atom, distance from " << vd1[0] << " " << vd1[1] << " " << vd1[2] << endl;
	for(i1=0;i1<system->atoms();i1++) {
	  cout << "distfrom " << time << " " << i1 << " " << system->dr(i1, vd1) << endl;
	}
      }

      // ******************************
      // Join
      // ******************************
      else if(arg=="join") {
	string dummy;
	Atom_System *other;

	argcheck(argc, i, 1);
        dummy = argv[i+1];
	i = i + 2;
	if(dummy_check) continue;

        // load other system
        other = new Atom_System();
	other->atypes = system->atypes;
	if(mode=="xyz") other->readxyz(dummy.c_str(), xyzin, 1);
	else other->readxyz(dummy.c_str(), 0, 1);

        cout << "Joining " << dummy << endl;
        system->join(*other);
        delete other;
      }

      // ******************************
      // On top
      // ******************************
      else if(arg=="ontop") {
	double t1, t2, t3;
	int i1;
	string dummy;
	valarray<double> dr(3);
	Atom_System *other;

	argcheck(argc, i, 2);
        t1 = atof(argv[i+1]);  // dist
        dummy = argv[i+2];
	i = i + 3;
	if(dummy_check) continue;

        // load other system
        other = new Atom_System();
	other->atypes = system->atypes;
	if(mode=="xyz") other->readxyz(dummy.c_str(), xyzin, 1);
	else other->readxyz(dummy.c_str(), 0, 1);

        // get highest atom in this system
        t2 = system->x0[2];
        for(i1=1;i1<system->atoms();i1++) {
          if(system->x0[3*i1 + 2] > t2) t2 = system->x0[3*i1 + 2];
        }

        // get lowest atom in other system
        t3 = other->x0[2];
        for(i1=1;i1<other->atoms();i1++) {
          if(other->x0[3*i1 + 2] < t3) t3 = other->x0[3*i1 + 2];
        }

        dr[0] = 0;
        dr[1] = 0;
        dr[2] = t2 - t3 + t1;

        cout << "Highest atom in this system: " << t2 << endl;
        cout << "Lowest atom in other system: " << t3 << endl;
        cout << "Lifting by:                  " << dr[2] << endl;
        cout << "Middle at:                   " << t2 + t1/2 << endl;

        other->translate(dr, true);
        system->join(*other);
        delete other;
      }

      // ******************************
      // ensuredist
      // ******************************
      else if(arg=="ensuredist") {
	double cut;

	argcheck(argc, i, 1);
	cut = atof(argv[i+1]);
	i = i + 2;
	if(dummy_check) continue;


	// --- variables

	int i, j, k;                 // loops
	double cutsq = cut*cut;      // cutoff^2
	double rmin;                 // minimum distance
	double f = 0.1;              // move s.t. dr -> (1 + f)dr


	// --- init

	cout << "Ensuring no atoms are closer than " << cut << " to each other..." << endl;
	cout << "  (if criterion too strict, might get stuck in (semi-)infinite loop)" << endl;

        system->init_nlist(cut,cut+0.0001);


	// --- start

	rmin = 0.0;
	while(rmin <= cut) {
	  int at, startnei;  // loop over neighbour list

	  rmin = cut + 1;

	  // loop over atoms
	  startnei = 0;
	  for(i=0;i<system->atoms();i++) {

	    // loop over neighbour list
	    for(j=startnei+1;j<=startnei + system->nlist[startnei];j++) {
	      double dist;  // distance between two atoms
	      valarray<double> dx(3);  // displacement vector

	      at = system->nlist[j];

	      // check
	      if(i==at) {
		cerr << "An atom is its own neighbour, impossible!" << endl;
		exit(-1);
	      }

	      // get distance
	      dx = system->dri(i, at, dist);

	      // move atom
	      if(dist < cut) {
		dx *= f/2.0;  // scale displacement
	      }
	      system->moveatom(i, dx);
	      system->moveatom(at, -dx);

	      if(dist*(1+f) < rmin) {
		rmin = dist*(1+f);
	      }

	      // assuming neighbour list will stay valid

	    }  // end of loop over neighbours
	    startnei += system->nlist[startnei] + 1;
	  }  // end of loop over atoms

	  if(system->update_already()) {
	    system->update_linkedlist();
	    system->update_nlist();
	  }
	}  // ok

	cout << "  ... done. " << endl;
      }

      // ******************************
      // Remove atoms in half space
      // ******************************
      else if(arg=="removehs") {
	valarray<double> vd1(3), dr(3);

	argcheck(argc, i, 6);
	vd1[0] = atof(argv[i+1]);
	vd1[1] = atof(argv[i+2]);
	vd1[2] = atof(argv[i+3]);
	dr[0] = atof(argv[i+4]);
	dr[1] = atof(argv[i+5]);
	dr[2] = atof(argv[i+6]);
	i = i + 7;
	if(dummy_check) continue;

        system->removehalfspace(vd1, dr);
      }

      // ******************************
      // Remove atoms below / above cut in dir
      // ******************************
      else if(arg=="remove") {
	int i1, i2;
	double t1;
	bool tb1;

	argcheck(argc, i, 3);
	i1 = atoi(argv[i+1]);
	i2 = atoi(argv[i+2]);
	t1 = atof(argv[i+3]);
	i = i + 4;
	if(dummy_check) continue;

        if(i2==1) tb1 = true;
        else tb1 = false;

        system->remove(i1, tb1, t1);
      }

      // ****************************************
      // Remove atom
      // ****************************************
      else if(arg=="removeat") {
	int i1, i2;

	argcheck(argc, i, 2);
	i1 = atoi(argv[i+1]);  // atom to remove
	i2 = atoi(argv[i+2]);  // mode
	i = i + 3;
	if(dummy_check) continue;

	cout << "Removing atom " << i1 << "...";
        system->removeat(i1, i2);
        cout << " ...done." << endl;
      }

      // ****************************************
      // Remove atoms of a certain type by number
      // ****************************************
      else if(arg=="removetype") {
	int i1;

	argcheck(argc, i, 1);
	i1 = atoi(argv[i+1]);
	i = i + 2;
	if(dummy_check) continue;

	cout << "Removing atoms of type " << i1 << "..." << endl;
        system->removetype(i1);
        cout << "...done." << endl;
      }

      // **************************************
      // Remove atoms of a certain type by name
      // **************************************
      else if(arg=="removename") {
	string dummy;

	argcheck(argc, i, 1);
	dummy = argv[i+1];
	i = i + 2;
	if(dummy_check) continue;

	cout << "Removing atoms of type " << dummy << "..." << endl;
        system->removetype(dummy);
        cout << "...done." << endl;
      }

      // ******************************
      // Remove atoms randomly
      // ******************************
      else if(arg=="removerand") {
	int i1;

	argcheck(argc, i, 1);
	i1 = atoi(argv[i+1]);  // number of atoms to remove
	i = i + 2;
	if(dummy_check) continue;

        system->removerand(i1);
      }

      // ******************************
      // Remove atoms based on coordination
      // ******************************
      else if(arg=="removecoord") {
	int i1, i2, i3, i4;
	double t1;

	argcheck(argc, i, 3);
        t1 = atof(argv[i+1]);  // cutoff
        i1 = atoi(argv[i+2]);  // min coordination to be removed
        i4 = atoi(argv[i+3]);  // max coordination to be removed
        i = i + 4;
	if(dummy_check) continue;

	// calculate number of neighbours and get statistics
        cout << "Removing atoms with coordination " << i1 << endl;

        system->init_nlist(t1,t1+0.0001);
        system->update_symmnlist();

	// remove atoms (use the K-array)
	for(i2=0;i2<system->atoms();i2++) {
	  system->block[i2] = system->symmnlist[i2][0];
	}

	i3 = 0;
	for(i2=0;i2<system->atoms();i2++) {
	  if(system->block[i2] >= i1 && system->block[i2] <= i4) {
	    system->removeat(i2);
	    i2--;
	    i3++;
	  }
	}

	// output statistics
	cout << "Removed " << i3 << " atoms." << endl;
      }

      // ******************************
      // Remove atoms based on distance from point
      // ******************************
      else if(arg=="removedist") {
	valarray<double> vd1(3);
	double t1;
	int i3, i2;

	argcheck(argc, i, 4);
        t1 = atof(argv[i+1]);  // distance
        vd1[0] = atof(argv[i+2]);  // point, x
        vd1[1] = atof(argv[i+3]);  // point, y
        vd1[2] = atof(argv[i+4]);  // point, z
        i = i + 5;
	if(dummy_check) continue;

        cout << "Removing atoms farther than " << t1 << " from " << vd1[0] << " " << vd1[1] << " " << vd1[2] << endl;

	i3 = 0;
	for(i2=0;i2<system->atoms();i2++) {
	  if(system->dr(i2, vd1) > t1) {
	    system->removeat(i2);
	    i2--;
	    i3++;
	  }
	}

	// output statistics
	cout << "Removed " << i3 << " atoms." << endl;
      }

      // ******************************
      // Remove atom depending on whether it's closer to a point than another
      // ******************************
      else if(arg=="removevor") {
	valarray<double> vd1(3), vd2(3);
	int i3, i2;
	double t1, t2;

	argcheck(argc, i, 6);
        vd1[0] = atof(argv[i+1]);  // point, x (keep if closer to vd1 than vd2)
        vd1[1] = atof(argv[i+2]);  // point, y
        vd1[2] = atof(argv[i+3]);  // point, z
        vd2[0] = atof(argv[i+4]);  // point, x
        vd2[1] = atof(argv[i+5]);  // point, y
        vd2[2] = atof(argv[i+6]);  // point, z
        i = i + 7;
	if(dummy_check) continue;

        cout << "Removing atoms closer to " << vd2[0] << " " << vd2[1] << " " << vd2[2] << " than " << vd1[0] << " " << vd1[1] << " " << vd1[2] << endl;

	i3 = 0;
	for(i2=0;i2<system->atoms();i2++) {
	  t1 = system->dr(i2, vd1);
	  t2 = system->dr(i2, vd2);
	  if(t2 < t1) {
	    system->removeat(i2);
	    i2--;
	    i3++;
	  }
	}

	// output statistics
	cout << "Removed " << i3 << " atoms." << endl;
      }

      // ******************************
      // Remove atoms based on distance from another atom
      // ******************************
      else if(arg=="removedat") {
	valarray<double> vd1(3);
	int i3, i2, i4, i1;
	double t1;

	argcheck(argc, i, 3);
        t1 = atof(argv[i+1]);  // distance
	i1 = atoi(argv[i+2]);  // atom
	i4 = atoi(argv[i+3]),  // min number of atoms
        i = i + 4;
	if(dummy_check) continue;

        vd1[0] = system->x0[system->dimension()*i1 + 0];  // point, x
        vd1[1] = system->x0[system->dimension()*i1 + 1];  // point, y
        vd1[2] = system->x0[system->dimension()*i1 + 2];  // point, z

        cout << "Removing atoms farther than " << t1 << " from atom " << i1 << " at " << vd1[0] << " " << vd1[1] << " " << vd1[2] << endl;

	i3 = 0;
	for(i2=0;i2<system->atoms();i2++) {
	  if(system->dr(i2, vd1) > t1) {
	    system->removeat(i2);
	    i2--;
	    i3++;
	  }
	  if(system->atoms()==i4) break;
	}

	// output statistics
	cout << "Removed " << i3 << " atoms." << endl;
      }

      // ******************************
      // Translate atoms with certain V
      // ******************************
      else if(arg=="moveV") {
        double t1, t2;
	valarray<double> dr(3);
	int i1;

	argcheck(argc, i, 5);
	t1 = atof(argv[i+1]);  // a
	t2 = atof(argv[i+2]);  // b
	dr[0] = atof(argv[i+3]);
	dr[1] = atof(argv[i+4]);
	dr[2] = atof(argv[i+5]);
	i = i + 6;
	if(dummy_check) continue;

	cout << "Translating atoms (including fixed ones) with V between " << t1 << " and " << t2 << " by ( " << dr[0] << " " << dr[1] << " " << dr[2] << " )..." << endl;

	for(i1=0;i1<system->atoms();i1++) {
	  if(system->V[i1] > t1 && system->V[i1] < t2) {
	    system->moveatom(i1, dr, true);
	  }
	}
      }

      // ******************************
      // Remove atoms with V[i]
      // ******************************
      else if(arg=="removeV") {
	double t1, t2;

	argcheck(argc, i, 2);
	t1 = atof(argv[i+1]);  // a
	t2 = atof(argv[i+2]);  // b
	i = i + 3;
	if(dummy_check) continue;

        system->removeV(t1, t2);
      }

      // ******************************
      // Remove atoms with K[i]
      // ******************************
      else if(arg=="removeK") {
	double t1, t2;

	argcheck(argc, i, 2);
	t1 = atof(argv[i+1]);  // a
	t2 = atof(argv[i+2]);  // b
	i = i + 3;
	if(dummy_check) continue;

        system->removeK(t1, t2);
      }

      // ******************************
      // Remove atoms to make sure there are not too close ones
      // ******************************
      else if(arg=="removetc") {
	double t1, t2;
	int i1, i2, i3;

	argcheck(argc, i, 1);
        t1 = atof(argv[i+1]);  // there should be no atoms closer than this to each other
        i = i + 2;
	if(dummy_check) continue;

        cout << "Removing atoms so that no two are closer than " << t1 << " from each other." << endl;

        system->init_nlist(t1,t1+0.0001);

	i3 = 0;
	for(i2=0;i2<system->atoms();i2++) {
	  i1 = system->find_closest_atom(i2, t2);
	  if(i1 != -1 && t2 < t1) {
	    system->removeat(i2);
	    system->update_linkedlist();
	    i2--;
	    i3++;
	  }
	}

	// output statistics
	cout << "Removed " << i3 << " atoms." << endl;
      }

      // ******************************
      // Remove atoms based on coordination
      // ******************************
      else if(arg=="removecurve") {
	valarray<double> p1, p2;  // coordinates of points on curve
	int d1, d2, s, n;
	int nremoved;             // number of removed atoms

	argcheck(argc, i, 6);
        d1 = atoi(argv[i+1]);     // direction one (0,1,2) = (x,y,z)
        d2 = atoi(argv[i+2]);     // direction two
        s = atoi(argv[i+3]);      // 1 = remove above
	n = atoi(argv[i+4]);      // number of points
	i = i + 5;
	p1.resize(n);
	p2.resize(n);
	for(int i1=0;i1<n;i1++) {
	  p1[i1] = atof(argv[i]);
	  p2[i1] = atof(argv[i + 1]);
	  i = i + 2;
	}
	if(dummy_check) continue;

	// checks
	if(d1 < 0 || d1 > 2 || d2 < 0 || d2 > 2) {
	  cerr << "removecurve: Directions should be 0, 1, 2 corresponding to x, y, z!" << endl;
	  exit(-1);
	}
	if(n==0) {
	  cerr << "removecurve: Curve has zero points!" << endl;
	  exit(-1);
	}

	// report
	if(s==1) cout << "Removing atoms above given curve with coordinates specified in directions " << d1 << "," << d2 << ", with (0,1,2)=(x,y,z)." << endl;
	else cout << "Removing atoms below given curve with coordinates specified in directions " << d1 << ", " << d2 << ", with (0,1,2)=(x,y,z)." << endl;

	// remove atoms
	nremoved = 0;
	for(int i2=0;i2<system->atoms();i2++) {
	  bool remove = false;           // remove this atom?
	  int ind;                       // atom is between p1[ind] and p1[ind+1] in direction d1
	  double c1, c2;                 // atom coordinates in direction d1, d2
	  double h;                      // height of curve at c1

	  // coordinates
	  c1 = system->x0[3*i2 + d1];
	  c2 = system->x0[3*i2 + d2];

	  // search for part of curve on which atom is
	  ind = -1;
	  for(int i3=0; i3<n-1; i3++) {
	    if(c1 >= p1[i3] && c1 < p1[i3+1]) {
	      ind = i3;
	      break;
	    }
	  }
	  // check
	  if(ind==-1) {
	    cerr << "removecurve: It seem atom " << i2 << " has coordinate " << c1 << " in direction " << d1 << " and the curve does not span this." << endl;
	    cerr << "  The points read in were:" << endl;
	    for(int i3=0; i3<n; i3++) {
	      cerr << "  " << p1[i3] << "  " << p2[i3] << endl;
	    }
	    exit(-1);
	  }

	  // height of curve
	  h = (c1 - p1[ind])/(p1[ind+1] - p1[ind])*(p2[ind+1] - p2[ind]) + p2[ind];

	  if( (s==1 && c2 > h) || (s!=1 && c2 < h) ) {
	    system->removeat(i2);
	    i2--;
	    nremoved++;
	  }
	}

	// output statistics
	cout << "Removed " << nremoved << " atoms." << endl;
      }

      // ******************************
      // Distance from (x,y) || z
      // ******************************
      else if(arg=="distz") {
	double t1, t2, t3;

	argcheck(argc, i, 2);
	t1 = atof(argv[i+1]);
	t2 = atof(argv[i+2]);
	i = i + 3;
	if(dummy_check) continue;

        t3 = system->dist_par_z(t1, t2);
        cout << "Distance of closest atom to line (" << t1 << "," << t2 << ")= " << t3 << endl;
      }

      // ******************************
      // Cluster
      // ******************************
      else if(arg=="cluster") {
	double t1;
	int i1;

	argcheck(argc, i, 1);
	t1 = atof(argv[i+1]);  // cutoff
	i = i + 2;
	if(dummy_check) continue;

        cout << "Clustering atoms..." << endl;
        system->init_nlist(t1, t1+0.01);
        system->update_symmnlist();
        system->cluster(t1);
        cout << "...done. Clusters: (cluster, natoms)" << endl;
        cout << "----------" << endl;
        for(i1=0;i1<system->blocknat.size();i1++) {
          cout << i1 << " " << system->blocknat[i1] << endl;
        }
        cout << "----------" << endl;
      }

      // ******************************
      // Leave only largest block
      // ******************************
      else if(arg=="leavelargest") {
	int i1;

	argcheck(argc, i, 0);
	i = i + 1;
	if(dummy_check) continue;

        i1 = system->largestblock();
        system->leaveblock(i1);
      }

      // ******************************
      // Remove largest block
      // ******************************
      else if(arg=="removelargest") {
	int i1;

	argcheck(argc, i, 0);
	i = i + 1;
	if(dummy_check) continue;

        i1 = system->largestblock();
        system->removeblock(i1);
      }

      // ******************************
      // Remove blocks of specific size
      // ******************************
      else if(arg=="removeblocksizes") {
	int i1, i3, i2;

	argcheck(argc, i, 2);
	i1 = atoi(argv[i+1]);  // remove blocks from with this many atoms...
	i3 = atoi(argv[i+2]);  // ...to this many atoms
	i = i + 3;
	if(dummy_check) continue;

	for(i2=0;i2<system->blocknat.size();i2++) {
	  if(system->blocknat[i2]>=i1 && system->blocknat[i2]<=i3) system->removeblock(i2);
	}
      }

      // ******************************
      // Cluster size distribution
      // ******************************
      else if(arg=="cldist") {
	int i2, i5, i6, i1;
	valarray<int> vi1;

	argcheck(argc, i, 0);
	i = i + 1;
	if(dummy_check) continue;

        // get sputtering yields
        i2 = system->atoms();  // Y_atoms
        i5 = 0;  // Y_clusters
        vi1.resize(system->atoms()+1);  // number of clusters of different sizes
        vi1 = 0;

        // sputtered clusters
        for(i1=0;i1<system->blocknat.size();i1++) {
          if(system->blocknat[i1] > 0) {
            // total number
            i5++;

            // different sizes
            vi1[system->blocknat[i1]]++;
          }
        }

        // check some numbers
        if(vi1.sum() != i5) {
          cerr << "Error / Bug: The number of clusters doesn't match: " << vi1.sum() << " " << i5 << endl;
          exit(-1);
        }
        i6 = 0;  // number of atoms from vi1
        for(i1=0;i1<vi1.size();i1++) {
          i6 += i1*vi1[i1];
        }
        if(system->atoms() != i6) {
          cerr << "Error / Bug: The number of atoms doesn't match: " << system->atoms() << " " << i6 << endl;
          exit(-1);
        }

        // cluster size distribution
        cout << "Outputting cluster size distribution: (cldist, time, #atoms, #clusters with #atoms atoms)" << endl;
        cout << "------------" << endl;
        i6 = 0;  // number of cluster with more than 4 atoms
        for(i1=0;i1<vi1.size();i1++) {
          if(vi1[i1] > 0) cout << "cldist " << time << " " << i1 << " " << vi1[i1] << endl;
          if(i1 > 4 && vi1[i1] > 0) i6 += vi1[i1];
        }
        cout << "------------" << endl;

        // output results
        cout << "Output: time Y_atoms Y_clusters Y_mono Y_di Y_tri Y_tetra Y_larger" << endl;
        cout << "cldistd " << time << " " << i2 << " " << i5;
        for(i1=1;i1<5;i1++) {
          if(vi1.size() > i1) cout << " " << vi1[i1];
          else cout << " " << 0;
        }
        cout << " " << i6 << endl;
      }

      // ******************************
      // Simulate AFM tip
      // ******************************
      else if(arg=="afm_tip") {
	Atom_System* tip;           // system for tip
	Atom_System* image;         // for storing height profile
	string tipf;                // tip file name
	double dg;                  // grid spacing
	valarray<int> pbc_orig(3);  // original pbc's
	valarray<int> pbc_zero(3);  // pbc's zero for this command
	for(int i1=0;i1<3;i1++) {
	  pbc_zero[i1] = 0;
	  pbc_orig[i1] = system->pbc[i1];
	}
	double cut;                 // cutoff for lowering tip
	double ztr;                 // highest z coord. of atoms in system
	double icutf;               // initial neighbour list cutoff = 2*cut*icutf

	argcheck(argc, i, 4);
	tipf = argv[i+1];        // xyz file containing tip
	dg = atof(argv[i+2]);    // grid spacing
	cut = atof(argv[i+3]);   // cutoff for lowering tip
	icutf = atof(argv[i+4]); // initial neighbour list cutoff factor
	i += 5;
	if(dummy_check) continue;


	// --- checks

	if(dg < 0) {
	  cerr << "afmh: Error: Negative grid spacing." << endl;
	  exit(-1);
	}


	// --- init

	// turn pbc's off
	system->setpbc(pbc_zero);

	// init neighbour list
	{
	  double temp = 2*cut*icutf;
	  system->init_nlist(temp, temp+0.0001);
	}

	// get translation to initial height of tip = "highest z-coord. in system" +  "2 * cut" - "lowest z-coord. in tip"
	tip = new Atom_System();
	tip->setpbc(pbc_zero);
	tip->readxyz(tipf.c_str(), 0, 1);
	ztr = tip->extrcoord(2, 0);
	delete tip;
	ztr = system->extrcoord(2, 1) + 2*cut - ztr;

	// init height profile
	image = new Atom_System();
	image->moreatoms(1000);
	// add atom type for grid point
	{
	  int temp = 1;
	  image->atypes.add("Gr", temp);
	}


	// --- start
	//
	//     Will loop over x and y with grid spacing dg. Assuming
	//     lowest point of tip at (x,y) = (0,0) as read from file,
	//     but deviation from this will only cause the picture to
	//     be displaced, apart from effects at the (non-periodic!)
	//     system edges.
	//
	//     At each (x,y), will lower tip to within "cut" of
	//     surface.
	//

	// loop over x
	for(double a = -system->box[0]/2 ; a <= system->box[0]/2 ; a += dg) {
	  // loop over y
	  for(double b = -system->box[1]/2 ; b <= system->box[1]/2 ; b += dg) {
	    double dist;  // closest tip-surface distance

	    cout << "afm_tip: Starting at (x,y) = (" << a << "," << b << ")." << endl;

	    // load tip
	    tip = new Atom_System();
	    tip->atypes = system->atypes;
	    tip->setpbc(pbc_zero);
	    tip->readxyz(tipf.c_str(), 0, 1);

	    // translate tip to initial position
	    {
	      valarray<double> temp(3);
	      temp[0] = a;
	      temp[1] = b;
	      temp[2] = ztr;
	      tip->translate(temp);
	    }

	    // loop over lowering tip
	    bool done = false;  // reached surface?
	    double prev_dist = -1.0;  // distance on previous step
	    while(!done) {
	      bool found = false;  // found at least one distance?

	      // get tip-surface distance
	      dist = 2*cut;
	      for(int i1=0;i1<tip->atoms();i1++) {
		int temp_index;
		double temp_dr;

		temp_index = system->find_closest_atom(tip->ri(i1), temp_dr);
		if( (temp_index >= 0 && temp_dr < dist) || (!found && temp_index >= 0) ) {
		  dist = temp_dr;
		  found = true;
		  //cout << "ppp found" << endl;
		}
		//cout << "ppp " << temp_index << " " << temp_dr << " " << dist << endl;
	      }
	      //cout << "dist " << dist << endl;

	      // check if not found
	      if(!found) {
		cerr << "afm_tip: Did not find tip-surface distance within cutoff, increase nlf parameter." << endl;
		system->join(*tip);
		system->savexyz("debug.xyz", 0, true);
		exit(-1);
	      }

	      // check that distance not increasing
	      if(prev_dist > 0 && dist > prev_dist) {
		cerr << "afm_tip: The tip-surface distance started increasing again. If you're using" << endl;
		cerr << "         a single-atom tip, the cutoff for lowering the tip is probably so" << endl;
		cerr << "         small, that the tip went through the substrate without touching it." << endl;
		exit(-1);
	      }
	      prev_dist = dist;

	      // check if done
	      if(dist < cut) done = true;

	      // if not done, lower tip
	      if(!done) {
		valarray<double> temp(3);
		temp[0] = 0.0;
		temp[1] = 0.0;
		temp[2] = -dist*0.99;
		tip->translate(temp);
		//cout << "tr dist " << temp[2] << " " << dist << endl;
	      }
	    }  // end of loop over lowering tip

	    // get height of tip at this position
	    {
	      valarray<double> temp(3);
	      temp[0] = a;
	      temp[1] = b;
	      temp[2] = tip->extrcoord(2, 0);
	      image->add_atom(temp, "Gr", 1000);
	    }

	    // delete tip
	    delete tip;

	  }  // end of loop over y
	}  // end of loop over x


	// --- translate lowest h to zero
	{
	  valarray<double> temp(3);
	  temp[0] = 0.0;
	  temp[1] = 0.0;
	  temp[2] = -image->extrcoord(2, 0);
	  image->translate(temp);
	}


	// --- output
	image->savexyz("afm_profile.xyz", 0, true);


	// --- restore pbc's to system and clean
	system->setpbc(pbc_orig);
	delete image;
      }

      //cout << "  surf <s d b file>             Output a x-y-z profile of the surface" << endl;
      // ******************************
      // Output surface profile (contributed by Nils Beckmann)
      //  (grid is a little approximate!)
      // ******************************
      else if(arg=="surf") {
	string surffile;	// output file name
	double s;		// grid spacing
	int d;	 		// direction to look at
	bool b;			// look for highest (=1) or lowest (=0)?
	int dgx,dgy;		// points in each direction

	double* surf;
	bool* found;
	ofstream file;
	int j, k, l;            // loops
	int d1,d2;		// the other directions

	// read parameters
	argcheck(argc, i, 4);
	s = atof(argv[i+1]);
	d = atoi(argv[i+2]);  // Nils atoi!
	// Nils: I changed the bool's: I hate it when integers are used, because although that works, it's a bit ambiguous.
	{
	  int temp;
	  temp = atoi(argv[i+3]);  // Nils: atoi instead of atof!
	  if(temp==0) b = false;
	  else if(temp==1) b = true;
	  else {
	    cerr << "surf: Parameter b (highest/lowest) should be 1/0." << endl;
	    exit(-1);
	  }
	}
	surffile = argv[i+4];
	i += 5;
	if(dummy_check) continue;

	// determine directions
	d1=0;
	d2=1;
	switch (d) {
	  case 0:
	    d1=1;
	    d2=2;
	  break;
	  case 1:
	    d1=0;
	    d2=2;
	  break;
	  case 2:
	    d1=0;
	    d2=1;
	  break;
	}

	// Nils: Why a factor of two?
	//       This will also not end up to a grid spacing of exactly s!
	//         -> You should define separate sx, sy after the next two lines
	//       Added floor(int)
	dgx = (int)floor(system->box[d1] / s * 2.0);
	dgy = (int)floor(system->box[d2] / s * 2.0);

	// Report
	cout << "Parameters: s=" << s << ", d=" << d << ", b=" << b << " " << endl;
	cout << "Surface Analysis, Grid Size: " << dgx << " x " << dgy << endl;

	// checks (Nils: < to <=)
	if((dgx <= 0)||(dgy <= 0)) {
	  cerr << "surf: Error: Negative or zero grid spacing." << endl;
	  exit(-1);
	}

	// Init
	surf = new double[dgx*dgy];
	found = new bool[dgx*dgy];
	for (j=0;j<dgx*dgy;j++) {
	  surf[j] = 0;
	  found[j] = false;
	}

	// get height profile of the surface
	// OLD IMPLEMENTION: loop over atoms times squares = slow
	// Nils: Removed. Using the linked list would make this
	// faster, but not as fast as what you implemented now.

	// reset data
	double exval; // saving the extremes
	if (b) exval = -9e9;
	else exval = 9e9;
	for (k=0;k<dgx;k++) {
	  for (l=0;l<dgy;l++) {
	    surf[k*dgy+l] = exval;
	  }
	}

	// get height profile of the surface
	for (j=0;j<system->atoms();j++) {
	  double cx, cy, cz;  // current coordinates // Nils: moved this inside for loop
	  int myx, myy;       // current grid cell
	  cx = system->x0[j*3+d1]+system->box[d1];
	  cy = system->x0[j*3+d2]+system->box[d2];
	  cz = system->x0[j*3+d];
	  myx = (int)floor(cx / s);  // Nils: added floor
	  myy = (int)floor(cy / s);
	  if ((myx<0)||(myx>=dgx)||(myy<0)||(myy>=dgy)) continue;
	  // Nils: Why not error instead of continue?

	  // Nils: If you wanted to have the grid points exact (see
	  // below), you could add a check here that the atom actually
	  // is within the grid point it's supposed to be.

	  if (b) {
	    if (cz > surf[myx*dgy+myy]) { 
	      surf[myx*dgy+myy] = cz;
	      found[myx*dgy+myy] = true;
	    }
	  }
	  else {
	    if (cz < surf[myx*dgy+myy]) { 
	      surf[myx*dgy+myy] = cz;
	      found[myx*dgy+myy] = true;
	    }
	  }
	}

	// set height of grid points with no atoms to zero
	for (k=0;k<dgx;k++) {
	  for (l=0;l<dgy;l++) {
	    if (!found[k*dgy+l]) surf[k*dgy+l] = 0.0;
	  }
	}

	// output
	file.open(surffile.c_str(), ios::trunc);
	assert(file);
	file << dgx*dgy << endl;
	file << "Surface Profile (direction=" << d << ", higher=" << b << ")" << endl;
	for (k=0;k<dgx;k++) {
	  for (l=0;l<dgy;l++) {
	    if (!found[k*dgy+l]) continue;
	    file << "P " << s*(k+0.5)-system->box[d1] << " " << s*(l+0.5)-system->box[d2] << " " << surf[k*dgy+l] << endl;
	    // Nils: I would prefer if the grid point limits were
	    // figured out exactly, so you could write that each
	    // output grid point gives the max height at the grid
	    // point +- 0.5*s in each direction. Just to be exact and
	    // be able to document nicely.
	  }
	}
	file.close();

	// clean
	delete [] surf;
	delete [] found;  // Nils: added this
      }

      // ******************************
      // Find vacancies by inserting spheres (only 3d)
      // ******************************
      else if(arg=="findvac") {
	double rs;  // radius of spheres
	double dr;  // grid spacing
	double t5, t6, t7;
	int j, k, l, i2, i3, i4, i1;
	bool tb1;
	valarray<int> vi1;
	valarray<double> vd1, vd2, vd3;
	Atom_System *other, *other2;
	string dummy;

	argcheck(argc, i, 2);
	rs = atof(argv[i+1]);  // radius of spheres

	// determine whether to remove vacuum part (=largest vacancy cluster) or not
	if(system->pbc[0]==1 && system->pbc[1]==1 && system->pbc[2]==1) {
	  tb1 = false;
	}
	else {
	  tb1 = true;
	}
	// invert behaviour for negative lattice constant
	if(rs < 0.0) {
	  rs = -rs;
	  if(tb1 == true) tb1 = false;
	  else tb1 = true;
	}

	dr = atof(argv[i+2]);  // dr, grid spacing
	i = i + 3;
	if(dummy_check) continue;

	cout << "Finding vacancies in system by inserting spheres." << endl;

	// center only for pbcs
	if(system->pbc[0]==0 && system->pbc[1]==0 && system->pbc[2]==0) {
	  cout << "  Centering system for maximum efficiency." << endl;
	  system->center();
	}

	// determine box size
	other = new Atom_System();  // set also for the vacancy system
	for(j=0;j<3;j++) {
	  if(system->pbc[j]==0) {
	    t5 = abs(system->extrcoord(j, 0));
	    t6 = abs(system->extrcoord(j, 1));
	    if(t5 < t6) t5 = t6;
	    system->box[j] = 2*t5 + 2*1.5*rs;
	    cout << "  Determined box size from atom positions (pbc 0): box[" << j << "] = " << system->box[j] << endl;
	  }
	  else {
	    cout << "  Determined box size from user input (pbc 1): box[" << j << "] = " << system->box[j] << endl;
	  }
	  other->box[j] = system->box[j];
	  other->pbc[j] = system->pbc[j];
	}

	// init linked list
	system->init_nlist(rs + 0.01, rs + 0.02);

	// init vacancy trial grid
	vi1.resize(3);  // grid size
	vd1.resize(3);  // grid spacing
	for(j=0;j<3;j++) {
	  vi1[j] = (int)ceil(system->box[j]/dr);
	  vd1[j] = system->box[j]/vi1[j];
	  if(system->pbc[j]==0) vi1[j]++;
	}
	cout << "  Using vacancy trial grid: " << vi1[0] << " x " << vi1[1] << " x " << vi1[2] << endl;

	// init temp system to survive with one
	// Atom_System::find_closest_atom() call
	// per loop.
	other2 = new Atom_System();
	*other2 = *system;

	// create vacancies
	i2 = 0;  // number of created vacancies
	vd3.resize(3);  // temp
	vd2.resize(3);  // last saved vacancy
	i3 = 0;  // guess hit
	i4 = 0;  // guess miss
	for(j=0;j<vi1[0];j++) {
	  for(k=0;k<vi1[1];k++) {
	    for(l=0;l<vi1[2];l++) {

	      // is there a vacancy here?
	      vd3[0] = -system->box[0]/2 + j*vd1[0];
	      vd3[1] = -system->box[1]/2 + k*vd1[1];
	      vd3[2] = -system->box[2]/2 + l*vd1[2];

	      // optim: check if it's not possible to have a vacancy here
	      if(i2 > 0) {
		t6 = 0;
		for(i1=0;i1<3;i1++) {
		  t5 = abs(vd3[i1]-vd2[i1]);
		  if(system->pbc[i1]==1 && t5 > system->box[i1]/2.0) t5 = system->box[i1] - t5;
		  t6 += t5*t5;
		}
		t6 = sqrt(t6);  // distance to previous grid point

		if(t7 + t6 < rs) {
		  i3++;  // hit
		  continue;
		}
	      }
	      // optim: save position of previous grid point
	      for(i1=0;i1<3;++i1) {
		vd2[i1] = vd3[i1];
	      }
	      i4++;  // miss

	      i1 = other2->find_closest_atom(vd3, t5);  // t5 -> dist to closest vacancy / atom
	      // save vacancy if a close by vacancy found
	      if(t5 > rs || i1 == -1) {
		other->add_atom(vd3, "Va", 10000);
		other2->add_atom(vd3, "Va", 10000);
		if(i2==0) {
		  other->init_nlist(rs + 0.01, rs + 0.02);
		}
		else {
		  other->add_to_linkedlist(i2);
		}
		other2->add_to_linkedlist(other2->atoms()-1);
		i2++;
	      }  // vacancy saved

	      t7 = t5;  // optim: distance to closest atom / vacancy
	    }
	  }
	}  // vacancies created

	cout << "Optimization guess hit miss: " << i3 << " " << i4 << endl;

	delete other2;

	// remove outside part
	for(j=0;j<3;j++) {
	  other->box[j] = system->box[j];
	  other->pbc[j] = system->pbc[j];
	}
	if(tb1 && i2>0) {
	  cout << "  Removing vacuum part (use negative a to prevent this)" << endl;
	  other->init_nlist(rs + 0.2, rs + 0.22);
	  other->update_symmnlist();
	  other->cluster(rs + 0.1);
	  i1 = other->largestblock();
	  other->removeblock(i1);
	}

	// report
	cout << "  Found " << i2 << " vacancies" << endl;
	cout << "  Saving:" << endl;
	cout << "    vacancies.xyz:            vacancies" << endl;
	cout << "    vacancies_and_system.xyz: vacancies and system" << endl;

	// save
	dummy = "__comment__";
	other->setcomment(dummy);
	if(frame==1) other->savexyz("vacancies.xyz", 3, true);
	else other->savexyz("vacancies.xyz", 3, false);
	other->join(*system);
	if(frame==1) other->savexyz("vacancies_and_system.xyz", 3, true);
	else other->savexyz("vacancies_and_system.xyz", 3, false);

	// done
	delete other;
      }

      // ******************************
      // Running average of V[i]-array
      // ******************************
      else if(arg=="runaveV") {
	int i1, i2, j;
	double t1, t2, t3, t4, t5, t6;

	argcheck(argc, i, 5);
	i1 = atoi(argv[i+1]);  // direction [0,2]
	t1 = atof(argv[i+2]);  // a
	t2 = atof(argv[i+3]);  // b
	t3 = atof(argv[i+4]);  // dx
	t4 = atof(argv[i+5]);  // delta
	i = i + 6;
	if(dummy_check) continue;

	// check
	if(i1 < 0 || i1 > 2) {
	  cerr << "runaveV: Error: direction not in [0,2]: " << i1 << endl;
	  exit(-1);
	}
	if(t2 <= t1) {
	  cerr << "runaveV: Error: b <= a, a, b: " << t1 << " " << t2 << endl;
	  exit(-1);
	}
	if(t3 <= 0) {
	  cerr << "runaveV: Error: dx = " << t3 << endl;
	  exit(-1);
	}
	if(t4 <= 0) {
	  cerr << "runaveV: Error: delta = " << t4 << endl;
	  exit(-1);
	}


	// loop over points
	t5 = t1;  // current point
	while(t5 <= t2) {

	  t6 = 0;  // average at this point
	  i2 = 0;  // atoms in this set

	  // loop over atoms
	  for(j=0;j<system->atoms();j++) {

	    // add point
	    if(system->x0[3*j + i1] < t5 + t4 && system->x0[3*j + i1] > t5 - t4) {
	      t6 += system->V[j];
	      i2++;
	    }
	  }

	  // calculate average, give 0 if no data
	  if(i2==0) {
	    cout << "runavev " << time << " " << t5 << " " << 0 << " (no data)" << endl;
	  }
	  else {
	    cout << "runavev " << time << " " << t5 << " " << t6/i2 << endl;
	  }

	  // move point
	  t5 += t3;
	}
      }


      // *************************
      // *** OUTPUT COMMANDS   ***
      // *************************


      // ******************************
      // Pad with atoms so that all frames have the same number
      // ******************************
      else if(arg=="pad") {
	int npad;
	valarray<double> pos(3);

	argcheck(argc, i, 4);
        npad = atoi(argv[i+1]);  // number of atoms requested
	pos[0] = atof(argv[i+2]);  // position
	pos[1] = atof(argv[i+3]);
	pos[2] = atof(argv[i+4]);
	i = i + 5;
	if(dummy_check) continue;

	// insert atoms
	int nat = system->atoms();
	if(nat < npad) {
	  for(int j=0; j<npad-nat; j++) {
	    system->add_atom(pos, "Pp", 1000);
	  }
	}
	else if(nat > npad) {
	  cerr << "Error: pad: Trying to pad system to " << npad << " atoms, but frame " << frame << " already has " << system->atoms() << " atoms." << endl;
	  exit(-1);
	}
      }

      // ******************************
      // Save xyz-file (outf-option in previous versions)
      // ******************************
      else if(arg=="savexyz") {
	int i1;
	string dummy;

	argcheck(argc, i, 2);
        dummy = argv[i+1];  // file name
	i1 = atoi(argv[i+2]);  // xyzmode
	i = i + 3;
	if(dummy_check) continue;

	if(frame==1) {
	  system->savexyz(dummy, i1, true);
	}
	else {
	  system->savexyz(dummy, i1, false);
	}
      }

      // ******************************
      // Save xyz-file for each frame
      // ******************************
      else if(arg=="savexyzfr") {
	int i1;
	string dummy;

	argcheck(argc, i, 2);
        dummy = argv[i+1];  // file name
	i1 = atoi(argv[i+2]);  // xyzmode
	i = i + 3;
	if(dummy_check) continue;

        dummy = dummy + stringify(frame) + ".xyz";
        cout << "Saving xyz-file " << dummy << endl;
	system->savexyz(dummy.c_str(), i1, true);
      }

      // ******************************
      // Save xyz-file for each frame with comment as filename
      // ******************************
      else if(arg=="savexyzfrc") {
	int i1;
	string dummy;

	argcheck(argc, i, 1);
	i1 = atoi(argv[i+1]);  // xyzmode
	i = i + 2;
	if(dummy_check) continue;

        dummy = system->get_comment() + ".xyz";
        cout << "Saving xyz-file " << dummy << endl;
	system->savexyz(dummy.c_str(), i1, true);
      }

      // ******************************
      // Save bgf-file for each frame
      // ******************************
      else if(arg=="savebgf") {
	int i1;
	string dummy, dummy2;

	argcheck(argc, i, 2);
        dummy = argv[i+1];  // file name
        i1 = atoi(argv[i+2]);  // mode
	i = i + 3;
	if(dummy_check) continue;

        dummy2 = "fr" + stringify(frame);
        cout << "Saving bgf-file " << dummy << " with DESCRP " << dummy2 << endl;

	if(frame==1) system->savebgf(dummy.c_str(), dummy2, i1, true);
	else system->savebgf(dummy.c_str(), dummy2, i1, false);
      }

      // ******************************
      // Save bgf-file with description
      // ******************************
      else if(arg=="savebgfd") {
	int i1;
	string dummy, dummy2;

	argcheck(argc, i, 3);
        dummy = argv[i+1];  // file name
        i1 = atoi(argv[i+2]);  // mode
	dummy2 = argv[i+3];  // DESCRP
	i = i + 4;
	if(dummy_check) continue;

        cout << "Saving bgf-file " << dummy << " with DESCRP " << dummy2 << endl;

	if(frame==1) system->savebgf(dummy.c_str(), dummy2, i1, true);
	else system->savebgf(dummy.c_str(), dummy2, i1, false);
      }

      // ******************************
      // Save Siesta fdf-file
      // ******************************
      else if(arg=="savesiesta") {
	string dummy;

	argcheck(argc, i, 1);
        dummy = argv[i+1];  // filename
	i = i + 2;
	if(dummy_check) continue;

        cout << "Saving Siesta fdf-file " << dummy << endl;
	system->atypes.setmasses(true);
	system->save_siesta_fdf(dummy);
      }

      // ******************************
      // Save poscar-file
      // ******************************
      else if(arg=="saveposcar" || arg=="saveposcars") {
	string dummy;
	int i1, j;
	valarray<int> vi1;

	argcheck(argc, i, 3);  // not sufficient!
        dummy = argv[i+1];  // file
        system->set_lattice_constant(atof(argv[i+2]));  // a
        i1 = atoi(argv[i+3]);  // number of types
        vi1.resize(i1);  // types
        for(j=0;j<i1;j++) {
          vi1[j] = atoi(argv[i+4+j]);  // types
        }
	i = i + 4 + i1;
	if(dummy_check) continue;

        cout << "Saving poscar-file " << dummy << endl;
	if(arg=="saveposcar") {
	  system->saveposcar(dummy, 0, vi1);
	}
	else if(arg=="saveposcars") {
	  system->saveposcar(dummy, 1, vi1);
	}
      }

      // ******************************
      // Save pov-file
      // ******************************
      else if(arg=="savepov" || arg=="savepovfr") {
	int i1;
	string dummy;
	double t1, t2;

	argcheck(argc, i, 4);
        dummy = argv[i+1];  // file name
        i1 = atoi(argv[i+2]);  // mode
        t1 = atof(argv[i+3]);  // min
        t2 = atof(argv[i+4]);  // max
	i = i + 5;
	if(dummy_check) continue;

        if(arg=="savepovfr") {
	  dummy = dummy + stringify(frame) + ".pov";
	}

        cout << "Saving pov-file " << dummy << endl;
	system->savepov(dummy.c_str(), i1, t1, t2, true);
      }

      // ******************************
      // Save lammps data-file
      // ******************************
      else if(arg=="savelmp") {
	int i1;
	string dummy;
	string headers = "";

	argcheck(argc, i, 2);
        dummy = argv[i+1];     // file name
	i1 = atoi(argv[i+2]);  // mode
	i = i + 3;
	if(dummy_check) continue;

	// report
	cout << "Saving LAMMPS data-file " << dummy << endl;

	// construct headers for bonds etc. if needed
	if(lmp_bonds > 0) {
	  headers += stringify(lmp_bond_types) + " bond types\n";
	  headers += stringify(lmp_bonds) + " bonds\n";
	}
	if(lmp_angles > 0) {
	  headers += stringify(lmp_angle_types) + " angle types\n";
	  headers += stringify(lmp_angles) + " angles\n";
	}
	if(lmp_dihedrals > 0) {
	  headers += stringify(lmp_dihedral_types) + " dihedral types\n";
	  headers += stringify(lmp_dihedrals) + " dihedrals\n";
	}

	// save file
	if(frame==1) {
	  system->savelammps(dummy, i1, true, headers);
	}
	else {
	  system->savelammps(dummy, i1, false, headers);
	}

	// include bonds etc.
	if(lmp_bonds > 0 || lmp_angles > 0 || lmp_dihedrals > 0) {
	  ofstream data;
	  data.open(dummy.c_str(), ios::app);
	  assert(data);

	  vector<string> loop(3);
	  loop[0] = "lmp_bonds.dat";
	  loop[1] = "lmp_angles.dat";
	  loop[2] = "lmp_dihedrals.dat";
	  vector<int> number(3);
	  number[0] = lmp_bonds;
	  number[1] = lmp_angles;
	  number[2] = lmp_dihedrals;

	  for(int j=0; j<3; j++) {
	    if(number[j] > 0) {
	      ifstream file;
	      cout << "  including " << loop[j] << endl;
	      file.open(loop[j].c_str());
	      assert(file);

	      while(!file.eof()) {
		string line;
		getline(file, line);
		data << line << endl;
	      }
	      file.close();
	    }
	  }

	  data.close();
	}
      }


      // *******************************************
      // *** HIDDEN COMMANDS, MAY CONTAIN ERRORS ***
      // *******************************************

      // ******************************
      // Save mdcore-file
      // ******************************
      else if(arg=="savemdcore") {
	string dummy;
	int i1;

	argcheck(argc, i, 2);
        dummy = argv[i+1];  // file name
	i1 = atoi(argv[i+2]);  // mode
	i = i + 3;
	if(dummy_check) continue;

        cout << "Saving mdcore file " << dummy << endl;
	system->savemdcore(dummy, i1);
      }

      // ******************************
      // Get T and other stuff
      // ******************************
      else if(arg=="temp") {
	double t1, t2, t3, t4, t5;

	argcheck(argc, i, 0);
	i = i + 1;
	if(dummy_check) continue;

	system->atypes.setmasses();
	system->updateT();
	system->angmom(t1, t2, t3, t4);
        t5 = system->rgyr();
 
        cout << endl;
        cout << "atoms:         " << system->atoms() << endl;
	cout << "T:             " << system->T << " K" << endl;
	cout << "L:             " << t1 << " au*Å^2/fs" << endl;  // wrong!
	cout << "f:             " << t2 << " 1/fs" << endl;
	cout << "J:             " << t3 << " au*Å^2" << endl;
	cout << "T(L):          " << t4 << " K" << endl;
        cout << "R_g:           " << t5 << " Å" << endl;
        t1 = system->cmekin();
        cout << "E_{k,CM}:      " << t1 << " eV" << endl;
        cout << "E_{k,CM}/at.:  " << t1/system->atoms() << " eV" << endl;
      }

      // ******************************
      // Average cross section
      // ******************************
      //cout << "  avecs <iter rsyl rcut>        Get average cross section by bombarding along z inside a cylinder" << endl;
      else if(arg=="avecs") {
	int i1, j;
	double t1, t2, t3, t4, t5, t6, t7, t8;

	argcheck(argc, i, 3);
	i1 = atoi(argv[i+1]);  // iterations
	t1 = atof(argv[i+2]);  // r_cylinder
	t2 = atof(argv[i+3]);  // r_cut
	i = i + 4;
	if(dummy_check) continue;

        cout << "Finding out average cross section by bombarding a cylinder (r " << t1 << ") with cutoff " << t2 << " and " << i1 << " iterations." << endl;
        cout << "  Outputting: hit/miss r_closest xrec yrec alpha beta gamma" << endl;
        cout << "----------------------------------------" << endl;
        for(j=0;j<i1;j++) {
          // angles
          t3 = 2.0*M_PI*genrand();  // alpha
          t4 = acos(1.0-2.0*genrand());  // beta
          t5 = 2.0*M_PI*genrand();  // gamma
          // recoil x,y
          t6 = t1;
          t7 = t1;
          while(sqrt(t6*t6 + t7*t7) > t1) {
            t6 = t1*(1.0 - 2.0*genrand());
            t7 = t1*(1.0 - 2.0*genrand());
          }

          // rotate randomly
          system->rotate_euler(t3, t4, t5);

          // find closest distance to recoil
          t8 = system->dist_par_z(t6, t7);

          if(t8 < t2) {
            cout << "Hit  ";
          }
          else {
            cout << "Miss ";
          }
          cout << t8 << " " << t6 << " " << t7 << " " << t3 << " " << t4 << " " << t5 << endl;
        }
        cout << "----------------------------------------" << endl;
      }

      // ******************************
      // Strain
      // ******************************
      else if(arg=="strain") {
	vector< valarray<double> > tmatrix;  // temp matrix (strain)
	int j, k, l;
	valarray<double> vd1, vd2;

	tmatrix.resize(3);
	for(j=0;j<3;j++) {
	  tmatrix[j].resize(3);
	}
	argcheck(argc, i, 10);
	system->set_lattice_constant(atof(argv[i+1]));
	i++;
	for(j=0;j<3;j++) {
	  for(k=0;k<3;k++) {
	    tmatrix[j][k] = atof(argv[i+1]);  // row by row
	    i++;
	  }
	}
	i++;
	if(dummy_check) continue;

	system->strain(tmatrix);

	// report new box size vectors
	/*
	cout << "New box size vectors:" << endl;
	for(j=0;j<system->dimension();j++) {
	  cout << "boxvector " << j << " ";
	  for(k=0;k<system->dimension();k++) {
	    cout << tmatrix[j][k] << " ";
	  }
	  cout << endl;
	}
	*/

	// report box size lengths and angles
	vd1.resize(3);  // length of box vector
	vd2.resize(3);  // a.b a.c b.c

	// lengths
	for(j=0;j<3;j++) {
	  vd1[j] = 0;
	  for(k=0;k<3;k++) {
	    vd1[j] += tmatrix[j][k]*tmatrix[j][k];
	  }
	  vd1[j] = sqrt(vd1[j]);
	}

	// a.b -> angles
	j = 0;
	k = 1;
	vd2[0] = 0;
	for(l=0;l<3;l++) {
	  vd2[0] += tmatrix[j][l]*tmatrix[k][l];
	}
	vd2[0] /= vd1[j]*vd1[k];
	vd2[0] = acos(vd2[0])*180/M_PI;

	// a.c -> angles
	j = 0;
	k = 2;
	vd2[1] = 0;
	for(l=0;l<3;l++) {
	  vd2[1] += tmatrix[j][l]*tmatrix[k][l];
	}
	vd2[1] /= vd1[j]*vd1[k];
	vd2[1] = acos(vd2[1])*180/M_PI;

	// b.c -> angles
	j = 1;
	k = 2;
	vd2[2] = 0;
	for(l=0;l<3;l++) {
	  vd2[2] += tmatrix[j][l]*tmatrix[k][l];
	}
	vd2[2] /= vd1[j]*vd1[k];
	vd2[2] = acos(vd2[2])*180/M_PI;

	cout << "CRYSTX " << vd1[0] << " " << vd1[1] << " " << vd1[2] << " " << vd2[0] << " " << vd2[1] << " " << vd2[2] << endl;
      }

      // ******************************
      // Unknown option / command
      // ******************************
      else {
        cerr << "Error: Unknown option / command: " << arg << endl;
        exit(-1);
      }

    }  // end of command line arguments


    // ******************************************
    // *** FINISH SOME COMMANDS AT LAST FRAME ***
    // ******************************************


    // --- art
    {
      bool useart = false;
      for(int j=0; j<argc ; j++) {
	if(argv[i]=="art") {
	  useart = true;
	  break;
	}
      }

      if(useart && frame==frames) {
	cout << "Outputting amplitude for r(t) for all atoms:" << endl;
	cout << "--------------------------------------------" << endl;
	for(int i1=0;i1<system->atoms();i1++) {
	  cout << "art  " << i1 << " " << rmax[i1] - rmin[i1] << endl;
	}
	cout << "--------------------------------------------" << endl;
      }
    }

    cout << endl << endl;

}  // end of commandloop()


void argcheck(int argc, int i, int req) {

  /*
    Check that there have been enough arguments.

    argc      number of command line params
    i         current i
    req       parameters required
  */


  if(i+req >= argc) {
    cerr << "Error: Not enough input arguments!" << endl;
    exit(-1);
  }

}
